/*
 -----------------------------------------------------------------------------
 This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
 Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
#ifndef __RailsContext_H__
#define __RailsContext_H__

//#include "OgreBuildSettings.h"
#include "OgreLogManager.h"
#include "OgrePlugin.h"
//#include "OgreFileSystemLayer.h"
#include "OgreOverlaySystem.h"

#include "SdkTrays.h"

#include "CSoundManager.h"

#include "OgreScene.h"
#include "CShortLineModel.h"
#include <android/log.h>

#include "android_URL.h"
#include "android_strings.h"
#include "CXMacDataTypes.h"

#include "CAchievements.h"
#include "CPrefs.h"
#include "RailsPrefsController.h"

#include "android_apk.h"

#include <time.h>

#include "java_bgnactivity.hpp"

#define MAX_RAILS_FREE_LEVEL (5)

#define MAX_LOW_RES_HEIGHT (700)

// Static plugins declaration section
// Note that every entry in here adds an extra header / library dependency
#ifdef OGRE_STATIC_LIB
#   ifdef USE_RTSHADER_SYSTEM
#       include "OgreRTShaderSystem.h"
#       include "RTShaderHelper.h"
#   endif // USE_RTSHADER_SYSTEM
#endif

#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
#include <android_native_app_glue.h>
#endif

#include "OIS.h"

//extern struct android_app *__gandroidapp__;
extern JNIEnv* gAndroidActivityEnv;
extern jobject  gActivityInstance;
extern jmethodID gShowTrainopediaPopup;
extern jmethodID gHideTrainopediaPopup;
extern Ogre::String gInternalDataPath;

namespace OgreBites
{
	/*=============================================================================
	| Base class responsible for setting up a common context for samples.
	| May be subclassed for specific sample types (not specific samples).
	| Allows one sample to run at a time, while maintaining a sample queue.
	=============================================================================*/
	
	class RailsContext :
		public Ogre::FrameListener,
		public Ogre::WindowEventListener,
        public OIS::MultiTouchListener,
		public SdkTrayListener
	{
	private:

		RailsContext()
		{
			mRoot = 0;
			mWindow = 0;
			mCurrentScene = 0;
			mOverlaySystem = 0;
			mScenePaused = false;
			mFirstRun = true;
			mLastRun = false;
			mInputMgr = 0;
			mTrayMgr = 0;
			mChaptersTrayMgr = 0;
			mChapter1TrayMgr = 0;
			mChapter2TrayMgr = 0;
			mChapter3TrayMgr = 0;
			mAchTrayMgr = 0;
			mCreditsTrayMgr = 0;
			mOptionsTrayMgr = 0;
			mLevelLockedTrayMgr = 0;
			mHighScoresTrayMgr = 0;
			mUnlockChaptersOfferTrayMgr = 0;
			mDebugTrayMgr = 0;
#ifdef USE_RTSHADER_SYSTEM
			mShaderGenerator	 = NULL;		
			mMaterialMgrListener = NULL;
#endif // USE_RTSHADER_SYSTEM
			
			mAchMaxOffset = 0;
			mIsAchScrolling = false;
			
			mAllChaptersUnlocked = false;
			mUnlockLevelsLockedByAchievemints = false;
			mShowRandomAchievementsInCompleteLevelView = false;
			
			mIsTrainopediaOnScreen = false;
			mEraseButtonPressed = false;
			mEraseButton = 0;
			
			mStartGameBtn = 0;
			
			_smallFontSize = 8;
			_midFontSize = 16;
			_bigFontSize = 24;
		}
		
		
	public:
			
		static RailsContext* getShared()
		{
			static RailsContext* _singleton = NULL;
			if (!_singleton)
			{
				_singleton = new RailsContext();
			}
		
			return _singleton;
		}
		
		virtual ~RailsContext() 
		{
			// delete _singleton;
			// _singleton = NULL;
		}
			
#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
		virtual void initAppForAndroid(Ogre::RenderWindow *window, struct android_app* app, OIS::MultiTouch *mouse, OIS::Keyboard *keyboard)
		{
			mWindow = window;
			mInputContext.mMultiTouch = mouse;
			mInputContext.mKeyboard = keyboard;
			
			if (gInternalDataPath.length() > 0)
			{
				//__android_log_print(ANDROID_LOG_VERBOSE, "RAILS", ">>>>>>>>>> gInternalDataPath: %s", gInternalDataPath.c_str());
			}
			else
			{
				//__android_log_print(ANDROID_LOG_VERBOSE, "RAILS", ">>>>>>>>>> gInternalDataPath: NULL");
				gInternalDataPath = Ogre::String("/data/data/com.belightsoft.rails/files");
				//__android_log_print(ANDROID_LOG_VERBOSE, "RAILS", ">>>>>>>>>> gInternalDataPath changed to: %s", gInternalDataPath);
			}
            
            if(app != NULL)
            {
                mAssetMgr = app->activity->assetManager;
                Ogre::ArchiveManager::getSingleton().addArchiveFactory( new Ogre::APKFileSystemArchiveFactory(app->activity->assetManager) );
                Ogre::ArchiveManager::getSingleton().addArchiveFactory( new Ogre::APKZipArchiveFactory(app->activity->assetManager) );
            }
		}
#endif

		virtual Ogre::RenderWindow* getRenderWindow()
		{
			return mWindow;
		}

		virtual COgreScene* getCurrentScene()
		{
			return mCurrentScene;
		}
		
		void showTrainopedia()
		{
			if (gShowTrainopediaPopup)
			{
				gAndroidActivityEnv->CallVoidMethod(gActivityInstance, gShowTrainopediaPopup);
				mIsTrainopediaOnScreen = true;
			}
			else
				__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate \"showTrainopediaPopup\"!");
		}
		
		void hideTrainopedia()
		{
			if (gHideTrainopediaPopup)
			{
				gAndroidActivityEnv->CallVoidMethod(gActivityInstance, gHideTrainopediaPopup);
				mIsTrainopediaOnScreen = false;
			}
			else
				__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate \"hideTrainopediaPopup\"!");
		}
		
		//--------------------------------------
		
		int getMaxUnlockedLevel()
		{
			int maxUnlockedLevel = 1 + CPrefs::sharedPrefs()->maxSuccessLevel();
			
			
			bool isCompletedTutorial = CAchievements::shared()->isCompletedTutorial();
			if (isCompletedTutorial == false)
				maxUnlockedLevel = 0;
			//if (!jIsChaptersUnlocked())
			//	maxUnlockedLevel = MIN(maxUnlockedLevel, MAX_RAILS_FREE_LEVEL);
			
			if (isAllChaptersUnlocked())
				maxUnlockedLevel = MAX_LEVEL;
			
			return maxUnlockedLevel;
		}
		
		const char* getChaptersBackgroundMatName()
		{
			static Ogre::String s_matName = "";
			if (s_matName.length() == 0)
			{
				Ogre::Viewport* vp = mWindow->getViewport(0);
				if (!jNeedsUseLowResTextures() && Ogre::ResourceGroupManager::getSingleton().resourceGroupExists("Overlays_Large"))
					s_matName = "Rails/ChaptersBack_large";
				else
					s_matName = "Rails/ChaptersBack_mid";
			}
			
			return s_matName.c_str();
		}
		
		const char* getChapterMapMatName(int chapter)
		{
			Ogre::String matName;
			if (matName.length() == 0)
			{
				Ogre::Viewport* vp = mWindow->getViewport(0);
				matName = "Rails/Chapter_" + Ogre::StringConverter::toString(chapter) + "_Map";
				if (!jNeedsUseLowResTextures() && Ogre::ResourceGroupManager::getSingleton().resourceGroupExists("Overlays_Large"))
					matName += "_large";
				else
					matName += "_mid";
			}
			
			return matName.c_str();
		}
		
		/*-----------------------------------------------------------------------------
		| Quits the current Scene and starts a new one.
		-----------------------------------------------------------------------------*/
		virtual void runScene(COgreScene* s)
		{
#if OGRE_PROFILING
            Ogre::Profiler* prof = Ogre::Profiler::getSingletonPtr();
            if (prof)
                prof->setEnabled(false);
#endif

			if (mCurrentScene)
			{
#ifdef USE_RTSHADER_SYSTEM
				if (mShaderGenerator)
					mShaderGenerator->removeAllShaderBasedTechniques(); // clear techniques from the RTSS
#endif
				mCurrentScene->_shutdown();    // quit current Scene
				mScenePaused = false;          // don't pause the next Scene
			}

			mWindow->removeAllViewports();                  // wipe viewports
			destroyDummyScene();

			if (s)
			{
				hideAllTrays();
				
#ifdef USE_RTSHADER_SYSTEM
				if (mShaderGenerator)
					s->setShaderGenerator(mShaderGenerator);
#endif
				
				s->setup(mWindow, mInputContext, mOverlaySystem);   // start new Scene
			}
#if OGRE_PROFILING
            if (prof)
                prof->setEnabled(true);
#endif

			mCurrentScene = s;
		}

		/*-----------------------------------------------------------------------------
		| This function initializes the render system and resources.
		-----------------------------------------------------------------------------*/
		virtual void initApp( COgreScene* initialScene = 0 )
		{
			createRoot();

            setup();
			
			if (!mFirstRun)
				recoverLastScene();
            else if (initialScene)
				runScene(initialScene);
            //mRoot->saveConfig();

			Ogre::Root::getSingleton().getRenderSystem()->_initRenderTargets();

			// Clear event times
			Ogre::Root::getSingleton().clearEventTimes();
		}
		
/*-----------------------------------------------------------------------------
		| This function closes down the application - saves the configuration then 
		| shutdowns.
		-----------------------------------------------------------------------------*/
		virtual void closeApp()
		{
#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
			shutdown();
#endif
		}
		
		void showOptionsMenu()
		{
			hideAllPopups();
			setupOptionsTrayMgr();
			mOptionsTrayMgr->showAll();
		}
		
		void showDebugMenu()
		{
			hideAllPopups();
			setupDebugTrayMgr();
			mDebugTrayMgr->showAll();
		}
		
		void showUnlockAllChaptersPurchaseOfferTrayMgr()
		{
			setupUnlockChaptersOfferTrayMgr();
			mUnlockChaptersOfferTrayMgr->showAll();
		}
		
		void showLevelLockedTrayMgr(int achIndex)
		{
			if (!mLevelLockedTrayMgr->hasWidget("LevelLockedBackPanel"))
				setupLevelLockedTrayMgr();
			
			OgreBites::DecorWidget* achievement = (OgreBites::DecorWidget*)mLevelLockedTrayMgr->getWidget("LevelLockedAchievementIcon");
			achievement->setMaterial("Achievements/achievement_" + Ogre::StringConverter::toString(achIndex));
			
			OgreBites::Label* lbl = (OgreBites::Label*)mLevelLockedTrayMgr->getWidget("LevelLocked_AchTitle");
			lbl->setWrappedText(CAchievements::shared()->localizedName(achIndex));
			lbl->getOverlayElement()->setTop(achievement->getOverlayElement()->getTop() - lbl->getOverlayElement()->getHeight() * (float)(lbl->getLinesCount() - 0.5));
			
			lbl = (OgreBites::Label*)mLevelLockedTrayMgr->getWidget("LevelLocked_AchDescription");
			lbl->setWrappedText(CAchievements::shared()->localizedPreEarnedText(achIndex));
  			lbl->getOverlayElement()->setTop(achievement->getOverlayElement()->getTop() + achievement->getOverlayElement()->getHeight() / 2.f - lbl->getOverlayElement()->getHeight() * (float)lbl->getLinesCount() / 2.f);
			
			mLevelLockedTrayMgr->showAll();
		}
        
		virtual bool isCurrentScenePaused()
		{
			if (mCurrentScene) return mScenePaused;
			return false;
		}
		
		virtual void railsGoesToBackground()
		{
			if (mCurrentScene && !mScenePaused && CShortLineModel::sharedModel()->gameMode() != gm_demo)
				CPrefs::sharedPrefs()->setWasStoppedAtTheGame();
			showPauseDialog();
		}
		
		virtual void showPauseDialog()
		{
			if (mCurrentScene && !mScenePaused)
			{
				mCurrentScene->onPressPauseButton();
			}
		}
		
		virtual void pauseCurrentScene()
		{
			if (mCurrentScene && !mScenePaused)
			{
				CPrefs::sharedPrefs()->setWasStoppedAtTheGame(false);
				mScenePaused = true;
				mCurrentScene->pause();
				mCurrentScene->writeSnapshot();
				
				if (mTrayMgr)
				{
					updateStartGameButtonCaption();
					showBackgrop();
					showMainWindow();
				}
			}
		}

		virtual void unpauseCurrentScene(bool fromLastSnapshot = true)
		{
			if (!mCurrentScene)
			{
				runScene(COgreScene::getShared());
			}
			
			if (mCurrentScene)// && mCurrentScene->isPaused())
			{
				if (fromLastSnapshot)// && CShortLineModel::sharedModel()->gameMode() == gm_demo)
				{
					if (CPrefs::isGameSnapshot())
						mCurrentScene->continueFromTheLastSnapshot();
					else if (CAchievements::shared()->isCompletedTutorial())
						mCurrentScene->startLevelWithIndex(1, true);
					else
						mCurrentScene->startTutorialLevel();
				}
				
				mScenePaused = false;
				//mCurrentScene->unpause(restart);
				
				hideAllTrays();
			}
		}
		
		virtual void showMainWindow()
		{
			if (mTrayMgr)
				mTrayMgr->showAll();
			
			jShowRegisterDialogIfNeeds();
		}
			
		/*-----------------------------------------------------------------------------
		| Processes frame started events.
		-----------------------------------------------------------------------------*/
		virtual bool frameStarted(const Ogre::FrameEvent& evt)
		{
			captureInputDevices();      // capture input

			// manually call Scene callback to ensure correct order
			return (mCurrentScene && !mScenePaused) ? mCurrentScene->frameStarted(evt) : true;
		}
			
		/*-----------------------------------------------------------------------------
		| Processes rendering queued events.
		-----------------------------------------------------------------------------*/
		virtual bool frameRenderingQueued(const Ogre::FrameEvent& evt)
		{
			if (mCurrentScene && !mScenePaused)
				return mCurrentScene->frameRenderingQueued(evt);
			
#ifdef GOOGLE_PLUS_ENABLED
			updateWidgetStates();
			
			if (BGNActivity_IsCloudLoadSuccess())
			{
				BGNActivity_ResetCloudLoadState();
				CPrefs::resyncUserDefaults();
				RailsPrefsController::reinitAllLevelsInfo();
				CAchievements::shared()->reinitFromPrefs();
				
				CPrefs::submitAllLiderboards();
				BGNActivity_SaveState();
			}
#endif

			mTrayMgr->frameRenderingQueued(evt);
			
			if (mIsAchScrolling)
			{
				if (mAchMaxOffset > 0)
				{
					
					{
						setAchScrollPercentage(getAchScrollPercentage() + mAchSign * mAchMaxOffset / mAchViewHeight);
						mAchMaxOffset--;
						if (mAchMaxOffset <= 0)
						{
							mIsAchScrolling = false;
							mAchMaxOffset = 0;
						}
					}
			
					
				}
			}
			
			if (mEraseButtonPressed)
			{
				timespec currTime;
				clock_gettime(CLOCK_MONOTONIC, &currTime);
				timespec df = diffTime(mEraseButtonPressedTime, currTime);
				
				setEraseProgressLabelString(jGetLocalizedFString("i_seconds_before_erasure", 3 - (int)df.tv_sec), Ogre::ColourValue(1, 0, 0, 1));
				if (mEraseButton && ((df.tv_sec >= 3 && df.tv_nsec > 0) || (mEraseButton->getState() != BS_DOWN)))
				{
					buttonHit(mEraseButton);
				}
			}
			
			return true;
		}
			
		/*-----------------------------------------------------------------------------
		| Processes frame ended events.
		-----------------------------------------------------------------------------*/
		virtual bool frameEnded(const Ogre::FrameEvent& evt)
		{
			// manually call Scene callback to ensure correct order
			if (mCurrentScene && !mScenePaused && !mCurrentScene->frameEnded(evt)) return false;
			// quit if window was closed
			if (mWindow->isClosed()) return false;
			// // go into idle mode if current Scene has ended
			// if (mCurrentScene && mCurrentScene->isDone()) runScene(0);

			return true;
		}

		/*-----------------------------------------------------------------------------
		| Processes window size change event. Adjusts mouse's region to match that
		| of the window. You could also override this method to prevent resizing.
		-----------------------------------------------------------------------------*/
		virtual void windowResized(Ogre::RenderWindow* rw)
		{
			// manually call Scene callback to ensure correct order
			if (mCurrentScene && !mScenePaused) mCurrentScene->windowResized(rw);
		}

		// window event callbacks which manually call their respective Scene callbacks to ensure correct order

		virtual void windowMoved(Ogre::RenderWindow* rw)
		{
			if (mCurrentScene && !mScenePaused) mCurrentScene->windowMoved(rw);
		}

		virtual bool windowClosing(Ogre::RenderWindow* rw)
		{
			if (mCurrentScene && !mScenePaused) return mCurrentScene->windowClosing(rw);
			return true;
		}

		virtual void windowClosed(Ogre::RenderWindow* rw)
		{
			if (mCurrentScene && !mScenePaused) mCurrentScene->windowClosed(rw);
		}

		virtual void windowFocusChange(Ogre::RenderWindow* rw)
		{
			if (mCurrentScene && !mScenePaused) mCurrentScene->windowFocusChange(rw);
		}

		// keyboard and mouse callbacks which manually call their respective Scene callbacks to ensure correct order

		virtual bool keyPressed(const OIS::KeyEvent& evt)
		{
			//if (mTrayMgr->isDialogVisible()) return true;  // ignore keypresses when dialog is showing
			return true;
		}
		

		virtual bool keyReleased(const OIS::KeyEvent& evt)
		{
			if (evt.key == OIS::KC_ESCAPE)
			{
				// if (mIsTrainopediaOnScreen)
				// {
				// 	hideTrainopedia();
				// 	//mIsTrainopediaOnScreen = false;
				// 	return true;
				// }
				
				if (mLevelLockedTrayMgr && mLevelLockedTrayMgr->areTraysVisible()) // should be at the top
				{
					mLevelLockedTrayMgr->hideAll();
					return true;
				}
				
				if (mUnlockChaptersOfferTrayMgr && mUnlockChaptersOfferTrayMgr->areTraysVisible())
				{
					mUnlockChaptersOfferTrayMgr->hideAll();
					return true;
				}
				
				if (mChapter1TrayMgr && mChapter1TrayMgr->areTraysVisible() || mChapter2TrayMgr && mChapter2TrayMgr->areTraysVisible() || mChapter3TrayMgr && mChapter3TrayMgr->areTraysVisible())
				{
					showChaptersTrayMgr();
					return true;
				}
					
				if (mChaptersTrayMgr && mChaptersTrayMgr->areTraysVisible())
				{
					mChaptersTrayMgr->hideAll();
					showMainWindow();
					return true;
				}
				
				if (mHighScoresTrayMgr && mHighScoresTrayMgr->areTraysVisible())
				{
					mHighScoresTrayMgr->hideAll();
					showMainWindow();
					return true;
				}
				
				if (mDebugTrayMgr && mDebugTrayMgr->areTraysVisible())
				{
					mDebugTrayMgr->hideAll();
					showMainWindow();
					return true;
				}
				
				if (mOptionsTrayMgr && mOptionsTrayMgr->areTraysVisible())
				{
					mOptionsTrayMgr->hideAll();
					showMainWindow();
					return true;
				}
				
				if (mCreditsTrayMgr && mCreditsTrayMgr->areTraysVisible())
				{
					mCreditsTrayMgr->hideAll();
					showMainWindow();
					return true;
				}
				
				if (mAchTrayMgr && mAchTrayMgr->areTraysVisible())
				{
					mAchTrayMgr->hideAll();
					showMainWindow();
					return true;
				}
				
				if (mCurrentScene && !mScenePaused)
				{
					pauseCurrentScene();
					return true;
				}
				else
				{
					jCloseRailsApplication();
				}
			}
			else if (evt.key == OIS::KC_APPS)
			{
				if (mCurrentScene && !mScenePaused)
				{
					mCurrentScene->onPressPauseButton();
				}
				else
					showOptionsMenu();
				
				return true;
			}
			else if (mCurrentScene && !mScenePaused)
				return mCurrentScene->keyReleased(evt);
			
			return false;
		}

		virtual bool touchMoved(const OIS::MultiTouchEvent& evt)
		{
			// if (mIsTrainopediaOnScreen)
			// 	return true;
			
			if (mUnlockChaptersOfferTrayMgr && mUnlockChaptersOfferTrayMgr->areTraysVisible() && mUnlockChaptersOfferTrayMgr->injectMouseMove(evt))
				return true;

			if (mCurrentScene && !mScenePaused)
				return mCurrentScene->touchMoved(evt);
			
			if (mLevelLockedTrayMgr && mLevelLockedTrayMgr->areTraysVisible() || mCreditsTrayMgr && mCreditsTrayMgr->areTraysVisible() || mHighScoresTrayMgr && mHighScoresTrayMgr->areTraysVisible())
			{
				return true;
			}
			
			if (mDebugTrayMgr && mDebugTrayMgr->areTraysVisible())
			{
				mDebugTrayMgr->injectMouseMove(evt);
				return true;
			}
			
			if (mOptionsTrayMgr && mOptionsTrayMgr->areTraysVisible())
			{
				mOptionsTrayMgr->injectMouseMove(evt);
				return true;
			}
			
			if (mAchTrayMgr && mAchTrayMgr->areTraysVisible() && mAchTrayMgr->injectMouseMove(evt))
			{
				Ogre::Vector2 cursorPos(evt.state.X.abs, evt.state.Y.abs);
				float offsetY = mAchPressedCoord.y - cursorPos.y;
				mAchMaxOffset += offsetY;
				mAchLastOffsetY = offsetY;
				
				
				setAchScrollPercentage(getAchScrollPercentage() + offsetY / mAchViewHeight);
				
				
				mAchPressedCoord = cursorPos;
				return true;
			}
			
			if (mTrayMgr && mTrayMgr->areTraysVisible() && mTrayMgr->injectMouseMove(evt))
				return true;
			
			if (mChaptersTrayMgr && mChaptersTrayMgr->areTraysVisible() && mChaptersTrayMgr->injectMouseMove(evt))
				return true;
			
			if (mChapter1TrayMgr && mChapter1TrayMgr->areTraysVisible() && mChapter1TrayMgr->injectMouseMove(evt))
				return true;
			if (mChapter2TrayMgr && mChapter2TrayMgr->areTraysVisible() && mChapter2TrayMgr->injectMouseMove(evt))
				return true;
			if (mChapter3TrayMgr && mChapter3TrayMgr->areTraysVisible() && mChapter3TrayMgr->injectMouseMove(evt))
				return true;
			
			return false;
		}

		virtual bool touchPressed(const OIS::MultiTouchEvent& evt)
		{
			// if (mIsTrainopediaOnScreen)
			// 	return true;
			
			if (mUnlockChaptersOfferTrayMgr && mUnlockChaptersOfferTrayMgr->areTraysVisible())
			{
				mUnlockChaptersOfferTrayMgr->injectMouseDown(evt);
				return true;
			}
			
			if (mCurrentScene && !mScenePaused)
				return mCurrentScene->touchPressed(evt);
			
			if (mLevelLockedTrayMgr && mLevelLockedTrayMgr->areTraysVisible() || mCreditsTrayMgr && mCreditsTrayMgr->areTraysVisible() || mHighScoresTrayMgr && mHighScoresTrayMgr->areTraysVisible())
			{
				return true;
			}
			
			if (mDebugTrayMgr && mDebugTrayMgr->areTraysVisible())
			{
				mDebugTrayMgr->injectMouseDown(evt);
				return true;
			}
			
			if (mOptionsTrayMgr && mOptionsTrayMgr->areTraysVisible())
			{
				mOptionsTrayMgr->injectMouseDown(evt);
				return true;
			}
			
			if (mAchTrayMgr && mAchTrayMgr->areTraysVisible() && mAchTrayMgr->injectMouseDown(evt))
			{
				mAchPressedCoord = Ogre::Vector2(evt.state.X.abs, evt.state.Y.abs);
				mAchMaxOffset = 0;
				mIsAchScrolling = false;
				//gettimeofday(&mAchScrollStart, NULL);
				clock_gettime(CLOCK_MONOTONIC, &mAchScrollStart);
				return true;
			}
			
			if (mTrayMgr && mTrayMgr->areTraysVisible() && mTrayMgr->injectMouseDown(evt))
				return true;
			
			if (mChaptersTrayMgr && mChaptersTrayMgr->areTraysVisible() && mChaptersTrayMgr->injectMouseDown(evt))
				return true;
			
			if (mChapter1TrayMgr && mChapter1TrayMgr->areTraysVisible() && mChapter1TrayMgr->injectMouseDown(evt))
				return true;
			if (mChapter2TrayMgr && mChapter2TrayMgr->areTraysVisible() && mChapter2TrayMgr->injectMouseDown(evt))
				return true;
			if (mChapter3TrayMgr && mChapter3TrayMgr->areTraysVisible() && mChapter3TrayMgr->injectMouseDown(evt))
				return true;
			
			return false;
		}
		
		timespec diffTime(timespec start, timespec end)
		{
			timespec temp;
			if ((end.tv_nsec-start.tv_nsec)<0) {
				temp.tv_sec = end.tv_sec-start.tv_sec-1;
				temp.tv_nsec = 1000000000+end.tv_nsec-start.tv_nsec;
			} else {
				temp.tv_sec = end.tv_sec-start.tv_sec;
				temp.tv_nsec = end.tv_nsec-start.tv_nsec;
			}
			return temp;
		}
		
		virtual bool touchReleased(const OIS::MultiTouchEvent& evt)
		{
			// if (mIsTrainopediaOnScreen)
			// {
			// 	hideTrainopedia();
			// 	//mIsTrainopediaOnScreen = false;
			// 	return true;
			// }
			
			if (mLevelLockedTrayMgr && mLevelLockedTrayMgr->areTraysVisible())
			{
				if (mCurrentScene && !mScenePaused)
				{
					pauseCurrentScene();
					showChaptersTrayMgr();
				}
				else
				{
					mLevelLockedTrayMgr->injectMouseUp(evt);
					mLevelLockedTrayMgr->hideAll();
				}
				
				return true;
			}
			
			if (mUnlockChaptersOfferTrayMgr && mUnlockChaptersOfferTrayMgr->areTraysVisible())
			{
				if (!mUnlockChaptersOfferTrayMgr->injectMouseUp(evt))
					mUnlockChaptersOfferTrayMgr->hideAll();
				return true;
			}
			
			if (mCurrentScene && !mScenePaused)
				return mCurrentScene->touchReleased(evt);
			
			if (mDebugTrayMgr && mDebugTrayMgr->areTraysVisible())
			{
				if (!mDebugTrayMgr->injectMouseUp(evt))
					mDebugTrayMgr->hideAll();
				return true;
			}
			
			if (mOptionsTrayMgr && mOptionsTrayMgr->areTraysVisible())
			{
				if (!mOptionsTrayMgr->injectMouseUp(evt))
					mOptionsTrayMgr->hideAll();
				return true;
			}
			
			if (mHighScoresTrayMgr && mHighScoresTrayMgr->areTraysVisible())
			{
				if (!mHighScoresTrayMgr->injectMouseUp(evt))
					mHighScoresTrayMgr->hideAll();
				return true;
			}
			
			if (mCreditsTrayMgr && mCreditsTrayMgr->areTraysVisible())
			{
				if (!mCreditsTrayMgr->injectMouseUp(evt))
					mCreditsTrayMgr->hideAll();
				return true;
			}
			
			if (mAchTrayMgr && mAchTrayMgr->areTraysVisible())
			{
				if (mAchTrayMgr->injectMouseUp(evt))
				{
					if (fabs(mAchLastOffsetY) < 4)
						return true;
					
					clock_gettime(CLOCK_MONOTONIC, &mAchScrollEnd);
					
					timespec df = diffTime(mAchScrollStart, mAchScrollEnd);
					if (df.tv_sec > 0)
						return true;
					if (df.tv_nsec > 0.5e9)
						return true;
					
					float diff = df.tv_nsec / (float)0.1e10;
					diff *= 40;
					mAchMaxOffset /= diff;
					mAchSign = (mAchMaxOffset >= 0) ? 1.f : -1.f;
					mAchMaxOffset = fabs(mAchMaxOffset);
					mIsAchScrolling = true;
					
					return true;
				}
				else
				{
					mAchTrayMgr->hideAll();
					return true;
				}
					
			}
			
			if (mTrayMgr && mTrayMgr->areTraysVisible() && mTrayMgr->injectMouseUp(evt))
				return true;
			
			if (mChaptersTrayMgr && mChaptersTrayMgr->areTraysVisible() && mChaptersTrayMgr->injectMouseUp(evt))
				return true;
			
			if (mChapter1TrayMgr && mChapter1TrayMgr->areTraysVisible() && mChapter1TrayMgr->injectMouseUp(evt))
				return true;
			if (mChapter2TrayMgr && mChapter2TrayMgr->areTraysVisible() && mChapter2TrayMgr->injectMouseUp(evt))
				return true;
			if (mChapter3TrayMgr && mChapter3TrayMgr->areTraysVisible() && mChapter3TrayMgr->injectMouseUp(evt))
				return true;
			
			return false;
		}
		
		virtual bool touchCancelled(const OIS::MultiTouchEvent& evt)
        {
			if (mCurrentScene && !mScenePaused)
				return mCurrentScene->touchCancelled(evt);
			
            return true;
        }
		
		bool NeedsShowRandomAchievementsInCompleteLevelView()
		{
#ifdef ENABLE_DEBUG_DIALOGS
			return mShowRandomAchievementsInCompleteLevelView;
#else
			return false;
#endif
		}
		
		bool isAllChaptersUnlocked()
		{
#ifdef ENABLE_DEBUG_DIALOGS
			return mAllChaptersUnlocked;
#else
			return false;
#endif
		}
		
		bool isUnlockedLevelsLockedByAchievemints()
		{
#ifdef ENABLE_DEBUG_DIALOGS
			return mUnlockLevelsLockedByAchievemints;
#else
			return false;
#endif
		}
		
		void buttonPressed(Button* button)
		{
			if (button->getName() == "Options_EraseButton")
			{
				clock_gettime(CLOCK_MONOTONIC, &mEraseButtonPressedTime);
				mEraseButtonPressed = true;
				
				setEraseProgressLabelString(jGetLocalizedFString("i_seconds_before_erasure", 3), Ogre::ColourValue(1, 0, 0, 1));
			}
		}

		void buttonHit(Button* button)
		{
			CSoundManager::sharedManager()->soundClick();
			
			if (button->getName() == "StartGameBtn")
			{
				unpauseCurrentScene(true);
			}
			else if (button->getName() == "BelightLogoButton")
			{
				goToURL("http://www.belightsoft.com/products/rails/");
			}
			else if (button->getName() == "CreditsButton")
			{
				setupCreditsTrayMgr();
				mCreditsTrayMgr->showAll();
			}
			else if (button->getName() == "TrainopediaButton")
			{
				showTrainopedia();
			}
			else if (button->getName() == "TwitterButton")
			{
				goToURL("https://twitter.com/railsgame");
			}
			else if (button->getName() == "FacebookButton")
			{
				goToURL("https://www.facebook.com/railsgame");
			}
#ifdef GOOGLE_PLUS_ENABLED
			else if (button->getName() == "GooglePlusButton")
			{
				if (BGNActivity_IsSignedIn())
					BGNActivity_ShowLeaderboards();//BGNActivity_StartSignOut();
				else
					BGNActivity_StartSignIn();
			}
			else if (button->getName() == "Options_GPlus")
			{
				if (BGNActivity_IsSignedIn())
					BGNActivity_StartSignOut();
				else
					BGNActivity_StartSignIn();
			}
#endif
			else if (button->getName() == "AchievementsButton")
			{
				setupAchievementsTrayMgr();
				mAchTrayMgr->showAll();
			}
			else if (button->getName() == "HighScoresButton")
			{
				setupHighScoresTrayMgr();
				mHighScoresTrayMgr->showAll();
			}
			else if (button->getName() == "OptionsButton")
			{
				showOptionsMenu();
			}
			else if (button->getName() == "Options_SoundButton")
			{
				CPrefs::sharedPrefs()->setSoundOn(!CPrefs::sharedPrefs()->isSoundOn());
				CSoundManager::sharedManager()->soundTrainArrive();
				setupOptionsTrayMgr();
			}
			else if (button->getName() == "Options_MusicButton")
			{
				bool isMusicOn = !CPrefs::sharedPrefs()->isMusicOn();
				CPrefs::sharedPrefs()->setMusicOn(isMusicOn);

				if (isMusicOn)
					CSoundManager::sharedManager()->playIntroMusic();
				else
					CSoundManager::sharedManager()->stopMusic();

				setupOptionsTrayMgr();
			}
			else if (button->getName() == "Options_EraseButton")
			{
				//удаляем ачивменты из локального хранилища
				timespec currTime;
				clock_gettime(CLOCK_MONOTONIC, &currTime);
				timespec df = diffTime(mEraseButtonPressedTime, currTime);
				
				if (df.tv_sec >= 3)
				{
					CAchievements::shared()->resetAllAchievements();
					RailsPrefsController::sharedController()->resetGameProgressPreferences();
					
					CPrefs::eraseCloudLiderboardsAndState();
					
					if (CPrefs::isGameSnapshot())
						CPrefs::sharedPrefs()->removeSnapshotFile();
					updateStartGameButtonCaption();
					setEraseProgressLabelString(jGetLocalizedString("Erased"), Ogre::ColourValue(1, 0, 0, 1));
					updateChaptersTrays();
				}
				else
				{
					setEraseProgressLabelString(jGetLocalizedString("To_erase_game_progress_hold_the_button_for_3_seconds"), Ogre::ColourValue(1, 1, 1, 1));
				}
				mEraseButtonPressed = false;
			}
			else if (button->getName() == "SelectLevelBtn")
			{
				setupChaptersTrayMgr();
				showChaptersTrayMgr();
			}
			else if (button->getName() == "TutorialButton")
			{
				if (!mCurrentScene)
				{
					runScene(COgreScene::getShared());
				}
				
				mCurrentScene->startTutorialLevel();
				unpauseCurrentScene(false);
			}
			else if (button->getName() == "Chapter_1_Button")
			{
				int maxUnlockedLevel = 1 + CPrefs::sharedPrefs()->maxSuccessLevel();
				bool isCompletedTutorial = CAchievements::shared()->isCompletedTutorial();
				
				if ((maxUnlockedLevel < 0 || !isCompletedTutorial) && !isAllChaptersUnlocked())
					return;
				
				showChapter_1_TrayMgr();
			}
			else if (button->getName() == "Chapter_2_Button")
			{
				int maxUnlockedLevel = 1 + CPrefs::sharedPrefs()->maxSuccessLevel();
				bool isCompletedTutorial = CAchievements::shared()->isCompletedTutorial();
				
				if ((maxUnlockedLevel < 21 || !isCompletedTutorial) && !isAllChaptersUnlocked())
					return;
				
				showChapter_2_TrayMgr();
			}
			else if (button->getName() == "Chapter_3_Button")
			{
				int maxUnlockedLevel = 1 + CPrefs::sharedPrefs()->maxSuccessLevel();
				bool isCompletedTutorial = CAchievements::shared()->isCompletedTutorial();
				
				if ((maxUnlockedLevel < 41 || !isCompletedTutorial) && !isAllChaptersUnlocked())
					return;
				
				showChapter_3_TrayMgr();
			}
			else if (button->getName() == "AchievementsBackBtn")
			{
				showMainWindow();
				mAchTrayMgr->hideAll();
			}
			else if (button->getName() == "ChaptersBackBtn")
			{
				showMainWindow();
				mChaptersTrayMgr->hideAll();
			}
			else if (button->getName() == "Chapter1BackBtn")
			{
				showChaptersTrayMgr();
			}
			else if (button->getName() == "Chapter2BackBtn")
			{
				showChaptersTrayMgr();
			}
			else if (button->getName() == "Chapter3BackBtn")
			{
				showChaptersTrayMgr();
			}
			else if (Ogre::StringUtil::startsWith(button->getName(), "Play_Chapter_", false))
			{
				int index = button->getTag();
				
				int maxUnlockedLevel = getMaxUnlockedLevel();
				
				if (index > maxUnlockedLevel)
					return;
				
				if (!jIsChaptersUnlocked() && !isAllChaptersUnlocked() && index > 1 && index % 5 == 1)
				{
					int threeStarsSuccesslevel = CPrefs::sharedPrefs()->maxThreeStarsSuccessLevel();
					if (index > (threeStarsSuccesslevel + 1))
					{
						showUnlockAllChaptersPurchaseOfferTrayMgr();
						return;
					}
				}
				
				int achievementIndex = CAchievements::shared()->achievementUnlockingLevel(index);
				if (achievementIndex != -1 && !isUnlockedLevelsLockedByAchievemints())
				{
					setupLevelLockedTrayMgr();
					showLevelLockedTrayMgr(achievementIndex);
					
					return;
				}
				
				if (!mCurrentScene)
				{
					runScene(COgreScene::getShared());
				}
				
				mCurrentScene->startLevelWithIndex(index, true);
				unpauseCurrentScene(false);
			}
			else if (button->getName() == "DemoBtn")
			{
				if (!mCurrentScene)
				{
					runScene(COgreScene::getShared());
				}
				
				mCurrentScene->startDemoLevel();
				unpauseCurrentScene(false);
			}
			else if (button->getName() == "BuyDialogButtonBuy")
			{
				mUnlockChaptersOfferTrayMgr->hideAll();
				jBuyUnlockChapters();
			}
			else if (button->getName() == "BuyDialogButtonBack")
			{
				if (mCurrentScene && !mScenePaused)
				{
					pauseCurrentScene();
					
					int threeStarsSuccesslevel = CPrefs::sharedPrefs()->maxThreeStarsSuccessLevel();
					if (threeStarsSuccesslevel < 21)
					{
						showChapter_1_TrayMgr();
					}
					else if (threeStarsSuccesslevel < 41)
					{
						showChapter_2_TrayMgr();
					}
					else if (threeStarsSuccesslevel < 61)
					{
						showChapter_3_TrayMgr();
					}
					else
					{
						showChaptersTrayMgr();
					}
					
					mUnlockChaptersOfferTrayMgr->hideAll();
				}
				else
					mUnlockChaptersOfferTrayMgr->hideAll();
			}
#ifdef ENABLE_DEBUG_DIALOGS
			else if (button->getName() == "ShowDebugDialog_Button")
			{
				showDebugMenu();
			}
			else if (button->getName() == "MM_ShowRandomAchievements")
			{
				mShowRandomAchievementsInCompleteLevelView = !mShowRandomAchievementsInCompleteLevelView;
				((OgreBites::GameButton*)button)->setMaterial(mShowRandomAchievementsInCompleteLevelView ? "Rails/check_on" : "Rails/check_off");
			}
			else if (button->getName() == "Chapters_UnlockCheck")
			{
				mAllChaptersUnlocked = !mAllChaptersUnlocked;
				
				((OgreBites::GameButton*)button)->setMaterial(isAllChaptersUnlocked() ? "Rails/check_on" : "Rails/check_off");
				
				updateChaptersTrays();
			}
			else if (button->getName() == "Chapters_IgnoreLBA")
			{
				mUnlockLevelsLockedByAchievemints = !mUnlockLevelsLockedByAchievemints;
				
				((OgreBites::GameButton*)button)->setMaterial(isUnlockedLevelsLockedByAchievemints() ? "Rails/check_on" : "Rails/check_off");
			}
			else if (button->getName() == "RegisterAndRate_EraseButton")
			{
				jResetRegisterAndRateDialogPrefs();
				BGNActivity_ShowToastMessage("Register and Rate info has been erased!");
			}
			else if (button->getName() == "Preferences_EraseButton")
			{
				CAchievements::shared()->resetAllAchievements();
				RailsPrefsController::sharedController()->resetGameProgressPreferences();
				updateChaptersTrays();
				BGNActivity_ShowToastMessage("Preferences has been erased!");
			}
			else if (button->getName() == "Snapshot_EraseButton")
			{
				if (CPrefs::isGameSnapshot())
					CPrefs::sharedPrefs()->removeSnapshotFile();
				updateStartGameButtonCaption();
				BGNActivity_ShowToastMessage("Snapshot has been erased!");
			}
#ifdef GOOGLE_PLUS_ENABLED
			else if (button->getName() == "CloudData_EraseButton")
			{
				if (BGNActivity_IsSignedIn())
				{
					CPrefs::eraseCloudLiderboardsAndState();
					BGNActivity_ShowToastMessage("Liderboards and Game State are erased from Cloud!");
				}
				else
				{
					BGNActivity_ShowToastMessage("Cloud data could not be erased. Please Sign in with Google Plus.");
				}
			}
#endif
#endif
		}
		
		void showChaptersTrayMgr()
		{
			setupChaptersTrayMgr();
			hideAllTrays();
			mChaptersTrayMgr->showAll();
		}
		
		void showChapter_1_TrayMgr()
		{
			setupChapter_1_TrayMgr();
			hideAllTrays();
			mChapter1TrayMgr->showAll();
		}
		
		void showChapter_2_TrayMgr()
		{
			setupChapter_2_TrayMgr();
			hideAllTrays();
			mChapter2TrayMgr->showAll();
		}
		
		void showChapter_3_TrayMgr()
		{
			setupChapter_3_TrayMgr();
			hideAllTrays();
			mChapter3TrayMgr->showAll();
		}
		
		void hideAllPopups()
		{
			if (mAchTrayMgr)
				mAchTrayMgr->hideAll();
			if (mOptionsTrayMgr)
				mOptionsTrayMgr->hideAll();
			if (mHighScoresTrayMgr)
				mHighScoresTrayMgr->hideAll();
			if (mCreditsTrayMgr)
				mCreditsTrayMgr->hideAll();
			if (mLevelLockedTrayMgr)
				mLevelLockedTrayMgr->hideAll();
			if (mUnlockChaptersOfferTrayMgr)
				mUnlockChaptersOfferTrayMgr->hideAll();
			
			if (mDebugTrayMgr)
				mDebugTrayMgr->hideAll();
		}
		
		void hideAllTrays()
		{
			if (mTrayMgr)
				mTrayMgr->hideAll();
			if (mAchTrayMgr)
				mAchTrayMgr->hideAll();
			if (mOptionsTrayMgr)
				mOptionsTrayMgr->hideAll();
			if (mHighScoresTrayMgr)
				mHighScoresTrayMgr->hideAll();
			if (mCreditsTrayMgr)
				mCreditsTrayMgr->hideAll();
			if (mChaptersTrayMgr)
				mChaptersTrayMgr->hideAll();
			if (mChapter1TrayMgr)
				mChapter1TrayMgr->hideAll();
			if (mChapter2TrayMgr)
				mChapter2TrayMgr->hideAll();
			if (mChapter3TrayMgr)
				mChapter3TrayMgr->hideAll();
			if (mLevelLockedTrayMgr)
				mLevelLockedTrayMgr->hideAll();
			if (mUnlockChaptersOfferTrayMgr)
				mUnlockChaptersOfferTrayMgr->hideAll();
			
			if (mDebugTrayMgr)
				mDebugTrayMgr->hideAll();
		}
		
		void updateChaptersTrays()
		{
			if (isChaptersTrayMgrInitialised())
				setupChaptersTrayMgr();
			if (mChapter1TrayMgr && mChapter1TrayMgr->areTraysVisible())
				setupChapter_1_TrayMgr();
			if (mChapter2TrayMgr && mChapter2TrayMgr->areTraysVisible())
				setupChapter_2_TrayMgr();
			if (mChapter3TrayMgr && mChapter3TrayMgr->areTraysVisible())
				setupChapter_3_TrayMgr();
		}

        bool isFirstRun() { return mFirstRun; }
        void setFirstRun(bool flag) { mFirstRun = flag; }
        bool isLastRun() { return mLastRun; }
        void setLastRun(bool flag) { mLastRun = flag; }
		
		const char* getFontNameForSize(int size, Ogre::String fontStyle)
		{
			Ogre::String fontName = "SdkTrays/";
			bool isJapan = jIsJapanLocale();
			fontStyle = isJapan ? "Caption" : fontStyle;
			
			if (size < (_smallFontSize + _midFontSize) / 2.f)
			{
				fontName = fontName + fontStyle;
			}
			else if (size < (_midFontSize + _bigFontSize) / 2.f)
			{
				fontName = fontName + "Mid" + fontStyle;
			}
			else
			{
				if (!_bigFontEnabled)
					fontName = fontName + "Mid" + fontStyle;
				else
					fontName = fontName + "Big" + fontStyle;
			}
			
			return fontName.c_str();
		}
		
	protected:
	
        /*-----------------------------------------------------------------------------
         | Sets up the context after configuration.
         -----------------------------------------------------------------------------*/
		virtual void setup()
		{
			static bool isFirstRun = true;
			
			if (isFirstRun)
			{
				isFirstRun = false;
	            if(mWindow == NULL)
	                mWindow = createWindow();
			
// #ifdef USE_RTSHADER_SYSTEM
// 			Ogre::RTShader::ShaderGenerator::initialize();
// 			mShaderGenerator = Ogre::RTShader::ShaderGenerator::getSingletonPtr();
// 			if (mShaderGenerator == NULL)
// 			{
// 				Ogre::LogManager::getSingleton().logMessage(">>>ERROR:  mShaderGenerator == NULL <<<");
// 			}
// 			else
// 			{
// 				// Create and register the material manager listener if it doesn't exist yet.
// 				if (mMaterialMgrListener == NULL)
// 				{
// 					mMaterialMgrListener = new ShaderGeneratorTechniqueResolverListener(mShaderGenerator);
// 					Ogre::MaterialManager::getSingleton().addListener(mMaterialMgrListener);
// 				}
// 			}
// #endif
				
				Ogre::TextureManager::getSingleton().setDefaultNumMipmaps(5);
			
				setupInput();
				locateResourcesFrom("essential.cfg", 1);
			
				Ogre::ResourceGroupManager::getSingleton().initialiseResourceGroup("Essential");
			
				createDummyScene();
				//initFonts(); // should be called after initialiseResourceGroup("Essential") and createDummyScene()
				initTrayMgr(); // should be called after initialiseResourceGroup("Essential") and createDummyScene() and initFonts()
			
	#ifdef USE_RTSHADER_SYSTEM
	            if(mRoot->getRenderSystem()->getCapabilities()->hasCapability(Ogre::RSC_FIXED_FUNCTION) == false)
	            {
	                Ogre::RTShader::ShaderGenerator::getSingletonPtr()->addSceneManager(mRoot->getSceneManager("DummyScene"));
	            }
	#endif // USE_RTSHADER_SYSTEM
			
				mWindow->update();
			
				if (mTrayMgr)
				{
					showBackgrop();
					mTrayMgr->showTrays();
					mWindow->update();
					mWindow->update();
					//mTrayMgr->showLoadingBar(1, 2, true);
				}
				
				// adds context as listener to process context-level (above the Scene level) events
				mRoot->addFrameListener(this);
				Ogre::WindowEventUtilities::addWindowEventListener(mWindow, this);
			}
			
			Ogre::Viewport* vp = mWindow->getViewport(0);
			int texturesLevel = 1;
			//if (vp != NULL && vp->getActualHeight() <= MAX_LOW_RES_HEIGHT)
			if (jNeedsUseLowResOverlays())
				texturesLevel = 2;
			
			//---------------------------------------------------------------------
#ifdef OBB_EXPANSOIN_FILE
			if (!jValidateMainOBB())
			{
				jHideProgress();
				jDownloadOBBPackage();
				return;
			}
#endif
			//---------------------------------------------------------------------
			
			
			//__android_log_print(ANDROID_LOG_INFO, "RailsNative", ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> texturesLevel: %i", texturesLevel);
			
			initFonts(); // should be called after initialiseResourceGroup("Essential") and createDummyScene()
			locateResourcesFrom("ui_resources.cfg", texturesLevel, true);
			loadResources();
			
		 	//if (mTrayMgr)
		 	//	mTrayMgr->hideLoadingBar();
			
			setupTrayMgr();
			hideAllTrays();
			
			//destroyDummyScene();
			
			windowResized(mWindow);   // adjust menus for resolution
			
			jOgreHasBeenStarted();
			if (CPrefs::sharedPrefs()->wasStoppedAtTheGame() && CPrefs::isGameSnapshot())
			{
				unpauseCurrentScene(true);
				mCurrentScene->onPressPauseButton();
			}
			else if (mTrayMgr)
			{
				mTrayMgr->showAll();
				CSoundManager::sharedManager()->playIntroMusic();
			}
			
			CPrefs::sharedPrefs()->setWasStoppedAtTheGame(false);
		}
		
		void initFonts()
		{
			_smallFontSize = 8;
			_midFontSize = 16;
			_bigFontSize = 24;
			_bigFontEnabled = false;
			
			Ogre::Viewport* vp = mWindow->getViewport(0);
			if (vp != NULL)
			{
				float height = vp->getActualHeight();
				if (height > MAX_LOW_RES_HEIGHT)
				{
					_smallFontSize = 24;
					_midFontSize = 36;
					_bigFontSize = 48;
				}
				
				if (height > MAX_LOW_RES_HEIGHT)
					_bigFontEnabled = true;
			}
			
			String codePoints = "33-126 161-255 339-339 1040-1105 8230-8230";
			String res = "96";
			
			if (!jIsJapanLocale())
			{
				//codePoints = jGetLocStringsCodePointRanges(); // начинаются проблемы во французской локализации
				createFont("SdkTrays/Caption", "Roboto-Regular.ttf", StringConverter::toString(_smallFontSize), res, codePoints);
				createFont("SdkTrays/MidCaption", "Roboto-Regular.ttf", StringConverter::toString(_midFontSize), res, codePoints);
				if (_bigFontEnabled)
					createFont("SdkTrays/BigCaption", "Roboto-Regular.ttf", StringConverter::toString(_bigFontSize), res, codePoints);
				
				createFont("SdkTrays/ICaption", "Roboto-Italic.ttf", StringConverter::toString(_smallFontSize), res, codePoints, "8");
				createFont("SdkTrays/MidICaption", "Roboto-Italic.ttf", StringConverter::toString(_midFontSize), res, codePoints, "8");
				if (_bigFontEnabled)
					createFont("SdkTrays/BigICaption", "Roboto-Italic.ttf", StringConverter::toString(_bigFontSize), res, codePoints, "8");
			
				createFont("SdkTrays/BCaption", "Roboto-Bold.ttf", StringConverter::toString(_smallFontSize), res, codePoints);
				createFont("SdkTrays/MidBCaption", "Roboto-Bold.ttf", StringConverter::toString(_midFontSize), res, codePoints);
				if (_bigFontEnabled)
					createFont("SdkTrays/BigBCaption", "Roboto-Bold.ttf", StringConverter::toString(_bigFontSize), res, codePoints);
			
				createFont("SdkTrays/BICaption", "Roboto-BoldItalic.ttf", StringConverter::toString(_smallFontSize), res, codePoints, "8");
				createFont("SdkTrays/MidBICaption", "Roboto-BoldItalic.ttf", StringConverter::toString(_midFontSize), res, codePoints, "8");
				if (_bigFontEnabled)
					createFont("SdkTrays/BigBICaption", "Roboto-BoldItalic.ttf", StringConverter::toString(_bigFontSize), res, codePoints, "8");
			}
			else
			{
				codePoints = jGetLocStringsCodePointRanges();
				
				createFont("SdkTrays/Caption", "DroidSansFallback.ttf", StringConverter::toString(_smallFontSize), res, codePoints);
				createFont("SdkTrays/MidCaption", "DroidSansFallback.ttf", StringConverter::toString(_midFontSize), res, codePoints);
				if (_bigFontEnabled)
					createFont("SdkTrays/BigCaption", "DroidSansFallback.ttf", StringConverter::toString(_bigFontSize), res, codePoints);
			}
			
		}
		
		void createFont(String fontName, String ttfName, String size, String res, String codePoints, String charSpacer = "5")
		{
			Ogre::FontManager& fontMgr = Ogre::FontManager::getSingleton();
			Ogre::ResourceManager::ResourceCreateOrRetrieveResult result = fontMgr.createOrRetrieve(fontName, "Essential");
			{
				//Ogre::ResourcePtr font = fontMgr.create(fontName, "Essential");
				Ogre::ResourcePtr font = result.first;
				font->setParameter("type", "truetype");
				font->setParameter("source", ttfName);
				font->setParameter("size", size);
				font->setParameter("resolution", res);
				font->setParameter("code_points", codePoints);
				font->setParameter("character_spacer", charSpacer);
				font->load();
			}
		}
		
		void initTrayMgr()
		{
			mTrayMgr = new SdkTrayManager("MainMenuTrayMgr", mWindow, mInputContext, this);
			mTrayMgr->hideCursor();
		}
		
		void setupTrayMgr()
		{
			if (!mStartGameBtn)
			{
				OgreBites::GameButton* btn = NULL;
				float f = (float)mWindow->getViewport(0)->getActualHeight() / 1536.f;
				f *= 1.2;
				
				int chSize = 56 * f;
				float koef = 0.34;
				mTrayMgr->createSeparator(TL_TOPRIGHT, "RightSeparator_1", 0, 35 * f, true);
				OgreBites::DecorWidget* railsLogo = mTrayMgr->createDecorWidget(TL_TOPRIGHT, "RailsLogo", "Rails/RailsLogo", f);
				mTrayMgr->createSeparator(TL_TOPRIGHT, "RightSeparator_2", 0, 10 * f, true);
				//mStartGameBtn = mTrayMgr->createStartMenuButton(TL_TOPRIGHT, "Rails/StartMenuButton", "StartGameBtn", jGetLocalizedString("New_Game"), f);
				mStartGameBtn = mTrayMgr->createGameButton(TL_TOPRIGHT, "StartGameBtn", "Rails/StartMenuButton", jGetLocalizedString("New_Game"), true, 620 * f, 126 * f);
				mTrayMgr->createSeparator(TL_TOPRIGHT, "RightSeparator_3", 0, 5 * f, true);
				mStartGameBtn->setCharHeight(chSize);
				mStartGameBtn->setCaptionPosition(0, - mStartGameBtn->getOverlayElement()->getHeight() * koef);
				mStartGameBtn->getTextAreaElement()->setFontName(getFontNameForSize(chSize, "BICaption"));
				btn = mTrayMgr->createGameButton(TL_TOPRIGHT, "SelectLevelBtn", "Rails/StartMenuButton", jGetLocalizedString("Select_Level"), true, 620 * f, 126 * f);
				mTrayMgr->createSeparator(TL_TOPRIGHT, "RightSeparator_4", 0, 5 * f, true);
				btn->setCharHeight(chSize);
				btn->setCaptionPosition(0, - btn->getOverlayElement()->getHeight() * koef);
				btn->getTextAreaElement()->setFontName(getFontNameForSize(chSize, "BICaption"));
				btn = mTrayMgr->createGameButton(TL_TOPRIGHT, "DemoBtn", "Rails/StartMenuButton", jGetLocalizedString("Demo"), true, 620 * f, 126 * f);
				btn->setCharHeight(chSize);
				btn->setCaptionPosition(0, - btn->getOverlayElement()->getHeight() * koef);
				btn->getTextAreaElement()->setFontName(getFontNameForSize(chSize, "BICaption"));
				mTrayMgr->setRightIndent(railsLogo->getOverlayElement()->getWidth() / 4);
				
				CXPoint pt = convertAbsCoordToScreen(CXPoint(1, 0.95));
				int bbW = 234 * f;
				int bbH = 256 * f;
				float bbOffsetX = 0.75;
				float textSize = 40 * f;
				Ogre::String fontName(getFontNameForSize(textSize, "ICaption"));
				btn = mTrayMgr->createGameButton(TL_NONE, "OptionsButton", "Rails/Options", jGetLocalizedString("Options"), true, bbW, bbH);
				btn->setCaptionHasPressedState(false);
				btn->getOverlayElement()->setPosition((int)pt.x - (int)((1.f + bbOffsetX) * bbW), (int)pt.y - 0.93 * bbH);
				btn->setCharHeight(textSize);
				btn->setCaptionPosition(0, btn->getOverlayElement()->getHeight() * 0.425);
				btn->getTextAreaElement()->setFontName(fontName);
				btn = mTrayMgr->createGameButton(TL_NONE, "HighScoresButton", "Rails/HighScores", jGetLocalizedString("High_Scores"), true, bbW, bbH);
				btn->setCaptionHasPressedState(false);
				btn->getOverlayElement()->setPosition((int)pt.x - (int)((2.f + bbOffsetX) * bbW), (int)pt.y - 0.93 * bbH);
				btn->setCharHeight(textSize);
				btn->setCaptionPosition(0, btn->getOverlayElement()->getHeight() * 0.425);
				btn->getTextAreaElement()->setFontName(fontName);
				btn = mTrayMgr->createGameButton(TL_NONE, "AchievementsButton", "Rails/Achievements", jGetLocalizedString("Achievements"), true, bbW, bbH);
				btn->setCaptionHasPressedState(false);
				btn->getOverlayElement()->setPosition((int)pt.x - (int)((3.f + bbOffsetX) * bbW), (int)pt.y - 0.93 * bbH);
				btn->setCharHeight(textSize);
				btn->setCaptionPosition(0, btn->getOverlayElement()->getHeight() * 0.425);
				btn->getTextAreaElement()->setFontName(fontName);
				
				btn = mTrayMgr->createGameButton(TL_NONE, "BelightLogoButton", "Rails/BelightLogo", "", true, 464 * f, 150 * f);
				
				pt = convertAbsCoordToScreen(CXPoint(0.05, 0));
				int beLogoH = btn->getOverlayElement()->getHeight();
				btn->getOverlayElement()->setPosition((int)(pt.x), (int)(0.5 * beLogoH));
				
				pt = convertAbsCoordToScreen(CXPoint(0.05, 0));
				btn = mTrayMgr->createGameButton(TL_NONE, "FacebookButton", "Rails/Facebook", "", true, 90 * f, 90 * f);
				btn->getOverlayElement()->setPosition((int)pt.x, (int)(1.4 * beLogoH));
				btn = mTrayMgr->createGameButton(TL_NONE, "TwitterButton", "Rails/Twitter", "", true, 90 * f, 90 * f);
				btn->getOverlayElement()->setPosition((int)pt.x + (int)(1.5 * btn->getOverlayElement()->getWidth()), (int)(1.4 * beLogoH));
#ifdef GOOGLE_PLUS_ENABLED
				btn = mTrayMgr->createGameButton(TL_NONE, "GooglePlusButton", "Rails/GPlus_Unsigned", "", true, 90 * f, 90 * f);
				btn->getOverlayElement()->setPosition((int)pt.x + (int)(3.75 * btn->getOverlayElement()->getWidth()), (int)(1.4 * beLogoH));
#endif
				
				btn = mTrayMgr->createGameButton(TL_NONE, "TrainopediaButton", "Rails/Trainopedia", jGetLocalizedString("Trainopedia"), true, 172 * f, 162 * f);
				btn->setCaptionHasPressedState(false);
				pt = convertAbsCoordToScreen(CXPoint(0.05, 0.95));
				btn->getOverlayElement()->setPosition((int)pt.x, (int)pt.y - btn->getOverlayElement()->getHeight());
				btn->setCharHeight(textSize);
				btn->setCaptionPosition(0, btn->getOverlayElement()->getHeight() * 0.5);
				btn->getTextAreaElement()->setFontName(fontName);
				
				btn = mTrayMgr->createGameButton(TL_NONE, "CreditsButton", "Rails/Credits", jGetLocalizedString("Credits"), true, 172 * f, 162 * f);
				btn->setCaptionHasPressedState(false);
				btn->getOverlayElement()->setPosition((int)pt.x + (int)(1.5 * btn->getOverlayElement()->getWidth()), (int)pt.y - btn->getOverlayElement()->getHeight());
				btn->setCharHeight(textSize);
				btn->setCaptionPosition(0, btn->getOverlayElement()->getHeight() * 0.5);
				btn->getTextAreaElement()->setFontName(fontName);

#ifdef ENABLE_DEBUG_DIALOGS
				btn = mTrayMgr->createGameButton(TL_TOP, "ShowDebugDialog_Button", "Rails/StartMenuButton", "Debug Dialog", true, 620 * f, 126 * f);
				btn->setCharHeight(chSize);
				btn->setCaptionPosition(0, - btn->getOverlayElement()->getHeight() * 0.34);
				btn->getTextAreaElement()->setFontName(getFontNameForSize(chSize, "BICaption"));
				mTrayMgr->setRightIndent(railsLogo->getOverlayElement()->getWidth() / 4);
#endif
								
				mTrayMgr->adjustTrays();
				
			}
			updateStartGameButtonCaption();
			
			if (!mChaptersTrayMgr)
			{
				mChaptersTrayMgr = new SdkTrayManager("ChaptersTrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mChapter1TrayMgr)
			{
				mChapter1TrayMgr = new SdkTrayManager("Chapter_1_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mChapter2TrayMgr)
			{
				mChapter2TrayMgr = new SdkTrayManager("Chapter_2_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mChapter3TrayMgr)
			{
				mChapter3TrayMgr = new SdkTrayManager("Chapter_3_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mAchTrayMgr)
			{
				mAchTrayMgr = new SdkTrayManager("MMAchievements_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mCreditsTrayMgr)
			{
				mCreditsTrayMgr = new SdkTrayManager("MMCredits_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mOptionsTrayMgr)
			{
				mOptionsTrayMgr = new SdkTrayManager("MMOptions_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mLevelLockedTrayMgr)
			{
				mLevelLockedTrayMgr = new SdkTrayManager("MMLevelLocked_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mHighScoresTrayMgr)
			{
				mHighScoresTrayMgr = new SdkTrayManager("MMHighScores_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mUnlockChaptersOfferTrayMgr)
			{
				mUnlockChaptersOfferTrayMgr = new SdkTrayManager("UnlockChaptersOffer_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mDebugTrayMgr)
			{
				mDebugTrayMgr = new SdkTrayManager("DebugDialog_TrayMgr", mWindow, mInputContext, this);
			}
			
		}
		
		CXPoint convertAbsCoordToScreen(CXPoint ip)
		{
			Ogre::Viewport* vp = mWindow->getViewport(0);
			//float scaleX = 1.f / ((3.f * (float)vp->getActualWidth()) / (4.f* (float)vp->getActualHeight()));
			float w = (float)vp->getActualWidth();
			float h = (float)vp->getActualHeight();
			
			ip.x *= w;
			ip.y *= h;
			
			return ip;
		}
		
#ifdef GOOGLE_PLUS_ENABLED
		void updateWidgetStates(bool force = false)
		{
			static bool s_signedIn = false;
			bool signedIn = BGNActivity_IsSignedIn();
			
			if (s_signedIn != signedIn || force)
			{
				s_signedIn = signedIn;
				if (mTrayMgr != NULL && mTrayMgr->hasWidget("GooglePlusButton"))
				{
					OgreBites::GameButton* btn = (OgreBites::GameButton*)mTrayMgr->getWidget("GooglePlusButton");
					btn->setMaterial(signedIn ? "Rails/GPlus_Liderboards" : "Rails/GPlus_Unsigned");
				}
				
				if (mOptionsTrayMgr != NULL && mOptionsTrayMgr->hasWidget("Options_GPlusCaption"))
				{
					OgreBites::Label* lbl = (OgreBites::Label*)mOptionsTrayMgr->getWidget("Options_GPlusCaption");
					lbl->setCaption(jGetLocalizedString(signedIn ? "sign_out" : "sign_in"));
					
					OgreBites::GameButton* btn = (OgreBites::GameButton*)mOptionsTrayMgr->getWidget("Options_GPlus");
					btn->setMaterial(signedIn ? "Rails/GPlus_Signed" : "Rails/GPlus_Unsigned");
				}
			}
		}
#endif
		
		void updateStartGameButtonCaption()
		{
			if (mStartGameBtn)
			{
				if (CPrefs::isGameSnapshot())
					mStartGameBtn->setCaption(jGetLocalizedString("Continue_Game"));
				else
					mStartGameBtn->setCaption(jGetLocalizedString("New_Game"));
			}
		}
		
		/*-----------------------------------------------------------------------------
		| Creates dummy scene to allow rendering GUI in viewport.
		-----------------------------------------------------------------------------*/
		virtual void createDummyScene()
		{
			mWindow->removeAllViewports();
			Ogre::SceneManager* sm = mRoot->createSceneManager(Ogre::ST_GENERIC, "DummyScene");
			sm->addRenderQueueListener(mOverlaySystem);
			Ogre::Camera* cam = sm->createCamera("DummyCamera");
			Ogre::Viewport* vp = mWindow->addViewport(cam);
			cam->setAutoAspectRatio(false);
			
//			return;
#ifdef USE_RTSHADER_SYSTEM
			
			vp->setMaterialScheme(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
			// Initialize shader generator.
			// Must be before resource loading in order to allow parsing extended material attributes.
			bool success = initializeRTShaderSystem(sm);
			if (!success) 
			{
				OGRE_EXCEPT(Ogre::Exception::ERR_FILE_NOT_FOUND, 
					"Shader Generator Initialization failed - Core shader libs path not found", 
					"RailsContext::createDummyScene");
			}
			if(mRoot->getRenderSystem()->getCapabilities()->hasCapability(Ogre::RSC_FIXED_FUNCTION) == false)
			{
				//newViewport->setMaterialScheme(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
				
				// creates shaders for base material BaseWhite using the RTSS
				Ogre::MaterialPtr baseWhite = Ogre::MaterialManager::getSingleton().getByName("BaseWhite", Ogre::ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);				
				baseWhite->setLightingEnabled(false);
				mShaderGenerator->createShaderBasedTechnique(
					"BaseWhite", 
					Ogre::MaterialManager::DEFAULT_SCHEME_NAME, 
					Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);	
			    mShaderGenerator->validateMaterial(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME, 
					"BaseWhite");
                if(baseWhite->getNumTechniques() > 1)
                {
				    baseWhite->getTechnique(0)->getPass(0)->setVertexProgram(
				    baseWhite->getTechnique(1)->getPass(0)->getVertexProgram()->getName());
				    baseWhite->getTechnique(0)->getPass(0)->setFragmentProgram(
				    baseWhite->getTechnique(1)->getPass(0)->getFragmentProgram()->getName());
                }

				// creates shaders for base material BaseWhiteNoLighting using the RTSS
				mShaderGenerator->createShaderBasedTechnique(
					"BaseWhiteNoLighting", 
					Ogre::MaterialManager::DEFAULT_SCHEME_NAME, 
					Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);	
			    mShaderGenerator->validateMaterial(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME, 
					"BaseWhiteNoLighting");
				Ogre::MaterialPtr baseWhiteNoLighting = Ogre::MaterialManager::getSingleton().getByName("BaseWhiteNoLighting", Ogre::ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
                if(baseWhite->getNumTechniques() > 1)
                {
				    baseWhiteNoLighting->getTechnique(0)->getPass(0)->setVertexProgram(
				    baseWhiteNoLighting->getTechnique(1)->getPass(0)->getVertexProgram()->getName());
				    baseWhiteNoLighting->getTechnique(0)->getPass(0)->setFragmentProgram(
				    baseWhiteNoLighting->getTechnique(1)->getPass(0)->getFragmentProgram()->getName());
                }
			}
#else
			vp->setMaterialScheme(Ogre::MaterialManager::DEFAULT_SCHEME_NAME);
#endif // USE_RTSHADER_SYSTEM
		}
        
		/*-----------------------------------------------------------------------------
		| Destroys dummy scene.
		-----------------------------------------------------------------------------*/
		virtual void destroyDummyScene()
		{
			if (mRoot->hasSceneManager("DummyScene"))
			{
				Ogre::SceneManager*  dummyScene = mRoot->getSceneManager("DummyScene");
#ifdef USE_RTSHADER_SYSTEM
				mShaderGenerator->removeSceneManager(dummyScene);
#endif
				dummyScene->removeRenderQueueListener(mOverlaySystem);
				mWindow->removeAllViewports();
				mRoot->destroySceneManager(dummyScene);
			}
		}
		
		/*-----------------------------------------------------------------------------
		| Creates the OGRE root.
		-----------------------------------------------------------------------------*/
		virtual void createRoot()
		{
			mRoot = Ogre::Root::getSingletonPtr();
			mOverlaySystem = OGRE_NEW Ogre::OverlaySystem();
		}


		/*-----------------------------------------------------------------------------
		| Creates the render window to be used for this context. I use an auto-created
		| window here, but you can also create an external window if you wish.
		| Just don't forget to initialise the root.
		-----------------------------------------------------------------------------*/
		virtual Ogre::RenderWindow* createWindow()
		{
			return mRoot->initialise(true);
		}

		/*-----------------------------------------------------------------------------
		| Sets up OIS input.
		-----------------------------------------------------------------------------*/
		virtual void setupInput(bool nograb = false)
		{
			// attach input devices
			windowResized(mWindow);    // do an initial adjustment of mouse area
			if(mInputContext.mMultiTouch)
				mInputContext.mMultiTouch->setEventCallback(this);
		}

		/*-----------------------------------------------------------------------------
		| Finds context-wide resource groups. I load paths from a config file here,
		| but you can choose your resource locations however you want.
		-----------------------------------------------------------------------------*/
		virtual void locateResourcesFrom(Ogre::String resFileName, int texturesLevel = 1, bool canAddOBB = false)
		{
			// load resource paths from config file
			Ogre::ConfigFile cf;
			cf.load(openAPKFile(resFileName.c_str(), mAssetMgr));

#ifdef OBB_EXPANSOIN_FILE
			if (texturesLevel == 1 && canAddOBB == true)
			{
				std::string obbPath = (texturesLevel == 1) ? jGetMountedOBBPath() : "";
				Ogre::ResourceGroupManager::getSingleton().addResourceLocation(obbPath, "Zip", "OBB_Section", true);
				Ogre::ResourceGroupManager::getSingleton().initialiseResourceGroup("OBB_Section");
			}
#endif
			
			Ogre::ConfigFile::SectionIterator seci = cf.getSectionIterator();
			Ogre::String sec, type, arch;

			// go through all specified resource groups
			while (seci.hasMoreElements())
			{
				sec = seci.peekNextKey();
				Ogre::ConfigFile::SettingsMultiMap* settings = seci.getNext();
				Ogre::ConfigFile::SettingsMultiMap::iterator i;

				// go through all resource paths
				for (i = settings->begin(); i != settings->end(); i++)
				{
					type = i->first;
					arch = i->second;
					
					// __android_log_print(ANDROID_LOG_INFO, "Rails", ">>>>>>>>>>>>>>arch: %s type: %s sec: %s", arch.c_str(), type.c_str(), sec.c_str());
					// /packs/RailsOverlay.zip type: APKZip sec: Overlays
					//Ogre::ResourceGroupManager::getSingleton().addResourceLocation(arch, type, sec);
					if (sec == "Overlays_Large" || sec == "Overlays_Mid")
					{
						if (sec == "Overlays_Large" && texturesLevel == 1)
						{
							Ogre::ResourceGroupManager::getSingleton().addResourceLocation(arch, type, sec);
						}
						else if (sec == "Overlays_Mid" && texturesLevel != 1)
						{
							Ogre::ResourceGroupManager::getSingleton().addResourceLocation(arch, type, sec);
						}
					}
					else
					{
						Ogre::ResourceGroupManager::getSingleton().addResourceLocation(arch, type, sec);
					}
				}
			}
		}

		/*-----------------------------------------------------------------------------
		| Loads context-wide resource groups. I chose here to simply initialise all
		| groups, but you can fully load specific ones if you wish.
		-----------------------------------------------------------------------------*/
		virtual void loadResources()
		{
			Ogre::ResourceGroupManager::getSingleton().initialiseResourceGroup("Overlays");
			Ogre::ResourceGroupManager::getSingleton().initialiseResourceGroup("General");
			//Ogre::ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
		}
		
		void showBackgrop()
		{
			mTrayMgr->showBackdrop("SdkTrays/RailsStartScreen");
			Ogre::Overlay* bdLayer = mTrayMgr->getBackdropLayer();
			
			Ogre::Viewport* vp = mWindow->getViewport(0);
			float scaleX = 1.f / ((3.f * (float)vp->getActualWidth()) / (4.f* (float)vp->getActualHeight()));
			bdLayer->setScale(1.f, 1.f/scaleX);
		}


		/*-----------------------------------------------------------------------------
		| Recovers the last Scene after a reset. You can override in the case that
		| the last Scene is destroyed in the process of resetting, and you have to
		| recover it through another means.
		-----------------------------------------------------------------------------*/
		virtual void recoverLastScene()
		{
		}

		/*-----------------------------------------------------------------------------
		| Cleans up and shuts down the context.
		-----------------------------------------------------------------------------*/
		virtual void shutdown()
		{
			if (mCurrentScene)
			{
				mCurrentScene->_shutdown();
				mCurrentScene = 0;
			}
			
#ifdef USE_RTSHADER_SYSTEM
			// Finalize the RT Shader System.
			finalizeRTShaderSystem();
#endif // USE_RTSHADER_SYSTEM
			
			// remove window event listener before shutting down OIS
			Ogre::WindowEventUtilities::removeWindowEventListener(mWindow, this);

			shutdownInput();
		}

		/*-----------------------------------------------------------------------------
		| Destroys OIS input devices and the input manager.
		-----------------------------------------------------------------------------*/
		virtual void shutdownInput()
		{
			// detach input devices
			windowResized(mWindow);    // do an initial adjustment of mouse area
			if(mInputContext.mKeyboard)
				mInputContext.mKeyboard->setEventCallback(NULL);
			if(mInputContext.mMouse)
				mInputContext.mMouse->setEventCallback(NULL);
#if OIS_WITH_MULTITOUCH
			if(mInputContext.mMultiTouch)
				mInputContext.mMultiTouch->setEventCallback(NULL);
#endif
			if(mInputContext.mAccelerometer)
                mInputContext.mAccelerometer->setEventCallback(NULL);
		}

		/*-----------------------------------------------------------------------------
		| Captures input device states.
		-----------------------------------------------------------------------------*/
		virtual void captureInputDevices()
		{
			mInputContext.capture();
		}
        
#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
        AAssetManager* mAssetMgr;       // Android asset manager to access files inside apk
#endif
		
#ifdef USE_RTSHADER_SYSTEM

		/*-----------------------------------------------------------------------------
		| Initialize the RT Shader system.	
		-----------------------------------------------------------------------------*/
		virtual bool initializeRTShaderSystem(Ogre::SceneManager* sceneMgr)
		{			
			if (Ogre::RTShader::ShaderGenerator::initialize())
			{
				mShaderGenerator = Ogre::RTShader::ShaderGenerator::getSingletonPtr();

				mShaderGenerator->addSceneManager(sceneMgr);

				// Create and register the material manager listener if it doesn't exist yet.
				if (mMaterialMgrListener == NULL) {
					mMaterialMgrListener = new ShaderGeneratorTechniqueResolverListener();
					Ogre::MaterialManager::getSingleton().addListener(mMaterialMgrListener);
				}
			}

			return true;
		}

		/*-----------------------------------------------------------------------------
		| Finalize the RT Shader system.	
		-----------------------------------------------------------------------------*/
		virtual void finalizeRTShaderSystem()
		{
			// Restore default scheme.
			Ogre::MaterialManager::getSingleton().setActiveScheme(Ogre::MaterialManager::DEFAULT_SCHEME_NAME);

			// Unregister the material manager listener.
			if (mMaterialMgrListener != NULL)
			{			
				Ogre::MaterialManager::getSingleton().removeListener(mMaterialMgrListener);
				delete mMaterialMgrListener;
				mMaterialMgrListener = NULL;
			}

			// Finalize RTShader system.
			if (mShaderGenerator != NULL)
			{				
				Ogre::RTShader::ShaderGenerator::finalize();
				mShaderGenerator = NULL;
			}
		}
#endif // USE_RTSHADER_SYSTEM
		
		bool isChaptersTrayMgrInitialised()
		{
			return (mChaptersTrayMgr && mChaptersTrayMgr->hasWidget("ChaptersBackBtn"));
		}
		
		void setupChaptersTrayMgr()
		{
			if (!mChaptersTrayMgr)
			{
				mChaptersTrayMgr = new SdkTrayManager("ChaptersTrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mChaptersTrayMgr->hasWidget("ChaptersBackBtn"))
			{
				mChaptersTrayMgr->setBackdropMaterial(getChaptersBackgroundMatName());
				Ogre::Overlay* bdLayer = mChaptersTrayMgr->getBackdropLayer();
				Ogre::Viewport* vp = mWindow->getViewport(0);
				float scaleX = 1.f / ((3.f * (float)vp->getActualWidth()) / (4.f* (float)vp->getActualHeight()));
				bdLayer->setScale(1.f, 1.f/scaleX);
			
				float f = (float)vp->getActualHeight() / 1536.f;
				f *= 1.2;
				float textSize = 50 * f;
				Ogre::String fontName(getFontNameForSize(textSize, "BICaption"));
				CXPoint pt = convertAbsCoordToScreen(CXPoint(0.5, 0.05));
				OgreBites::GameButton* btn = NULL;
				btn = mChaptersTrayMgr->createGameButton(TL_NONE, "TutorialButton", "Rails/Tutorial", jGetLocalizedString("Tutorial"), true, 396 * f, 396 * f);
				btn->setCaptionHasPressedState(false);
				pt.x -= btn->getOverlayElement()->getWidth() / 2;
				btn->getOverlayElement()->setPosition((int)(pt.x), (int)(pt.y));
				btn->setCharHeight(textSize);
				btn->setCaptionPosition(0, btn->getOverlayElement()->getHeight() * 0.4);
				btn->getTextAreaElement()->setFontName(fontName);
				
				pt = convertAbsCoordToScreen(CXPoint(0.5, 0.4));
				btn = mChaptersTrayMgr->createGameButton(TL_NONE, "Chapter_1_Button", "Rails/Chapter_1", jGetLocalizedString("Chapter_1"), true, 462 * f, 462 * f);
				btn->setCaptionHasPressedState(false);
				pt.x -= btn->getOverlayElement()->getWidth() / 2;
				btn->getOverlayElement()->setPosition((int)(pt.x) - (int)(1.25 * btn->getOverlayElement()->getWidth()), (int)(pt.y));
				btn->setCharHeight(textSize);
				btn->setCaptionPosition(0, btn->getOverlayElement()->getHeight() * 0.4);
				btn->getTextAreaElement()->setFontName(fontName);
				btn->setMaskMaterial("Rails/Chapters_lock");
				btn->setMaskOffsetAndSize(0, btn->getOverlayElement()->getHeight() * 0.54, 462 * f, 166 * f);
				btn->setFill(1);
				
				btn = mChaptersTrayMgr->createGameButton(TL_NONE, "Chapter_2_Button", "Rails/Chapter_2", jGetLocalizedString("Chapter_2"), true, 462 * f, 462 * f);
				btn->setCaptionHasPressedState(false);
				btn->getOverlayElement()->setPosition((int)(pt.x), (int)(pt.y));
				btn->setCharHeight(textSize);
				btn->setCaptionPosition(0, btn->getOverlayElement()->getHeight() * 0.4);
				btn->getTextAreaElement()->setFontName(fontName);
				btn->setMaskMaterial("Rails/Chapters_lock");
				btn->setMaskOffsetAndSize(0, btn->getOverlayElement()->getHeight() * 0.54, 462 * f, 166 * f);
				btn->setFill(1);
				
				btn = mChaptersTrayMgr->createGameButton(TL_NONE, "Chapter_3_Button", "Rails/Chapter_3", jGetLocalizedString("Chapter_3"), true, 462 * f, 462 * f);
				btn->setCaptionHasPressedState(false);
				btn->getOverlayElement()->setPosition((int)(pt.x) + (int)(1.25 * btn->getOverlayElement()->getWidth()), (int)(pt.y));
				btn->setCharHeight(textSize);
				btn->setCaptionPosition(0, btn->getOverlayElement()->getHeight() * 0.4);
				btn->getTextAreaElement()->setFontName(fontName);
				btn->setMaskMaterial("Rails/Chapters_lock");
				btn->setMaskOffsetAndSize(0, btn->getOverlayElement()->getHeight() * 0.54, 462 * f, 166 * f);
				btn->setFill(1);
			
				pt = convertAbsCoordToScreen(CXPoint(0.5, 0.95));
				btn = mChaptersTrayMgr->createGameButton(TL_NONE, "ChaptersBackBtn", "Rails/BackButton", "", true, 218 * f, 126 * f);
				pt.x -= btn->getOverlayElement()->getWidth() / 2;
				pt.y -= btn->getOverlayElement()->getHeight() / 2;
				btn->getOverlayElement()->setPosition((int)(pt.x), (int)(pt.y));
			}
			
			{
				int maxUnlockedLevel = getMaxUnlockedLevel();
				bool isCompletedTutorial = CAchievements::shared()->isCompletedTutorial();
				
				// if (!jIsChaptersUnlocked())
				// 	maxUnlockedLevel = MIN(maxUnlockedLevel, MAX_RAILS_FREE_LEVEL);
				
				OgreBites::GameButton* btn1 = (OgreBites::GameButton*)mChaptersTrayMgr->getWidget("Chapter_1_Button");
				OgreBites::GameButton* btn2 = (OgreBites::GameButton*)mChaptersTrayMgr->getWidget("Chapter_2_Button");
				OgreBites::GameButton* btn3 = (OgreBites::GameButton*)mChaptersTrayMgr->getWidget("Chapter_3_Button");
				if ((maxUnlockedLevel < 0 || !isCompletedTutorial) && !isAllChaptersUnlocked())
				{
					//btn1->setMaterial("Rails/Chapter_1_locked");
					btn1->setFill(1);
					btn1->setHasPressedState(false);
					//btn2->setMaterial("Rails/Chapter_2_locked");
					btn2->setFill(1);
					btn2->setHasPressedState(false);
					//btn3->setMaterial("Rails/Chapter_3_locked");
					btn3->setFill(1);
					btn3->setHasPressedState(false);
				}
				else if ((maxUnlockedLevel < 21 || !isCompletedTutorial) && !isAllChaptersUnlocked())
				{
					//btn1->setMaterial("Rails/Chapter_1");
					btn1->setFill(0);
					btn1->setHasPressedState(true);
					btn1->setCaptionHasPressedState(false);
					//btn2->setMaterial("Rails/Chapter_2_locked");
					btn2->setFill(1);
					btn2->setHasPressedState(false);
					//btn3->setMaterial("Rails/Chapter_3_locked");
					btn3->setFill(1);
					btn3->setHasPressedState(false);
				}
				else if ((maxUnlockedLevel < 41 || !isCompletedTutorial) && !isAllChaptersUnlocked())
				{
					//btn1->setMaterial("Rails/Chapter_1");
					btn1->setFill(0);
					btn1->setHasPressedState(true);
					btn1->setCaptionHasPressedState(false);
					//btn2->setMaterial("Rails/Chapter_2");
					btn2->setFill(0);
					btn2->setHasPressedState(true);
					btn2->setCaptionHasPressedState(false);
					//btn3->setMaterial("Rails/Chapter_3_locked");
					btn3->setFill(1);
					btn3->setHasPressedState(false);
				}
				else
				{
					//btn1->setMaterial("Rails/Chapter_1");
					btn1->setFill(0);
					btn1->setHasPressedState(true);
					btn1->setCaptionHasPressedState(false);
					//btn2->setMaterial("Rails/Chapter_2");
					btn2->setFill(0);
					btn2->setHasPressedState(true);
					btn2->setCaptionHasPressedState(false);
					//btn3->setMaterial("Rails/Chapter_3");
					btn3->setFill(0);
					btn3->setHasPressedState(true);
					btn3->setCaptionHasPressedState(false);
				}
			}
		}
		
		void updateLevelsForChapter(int chapter)
		{
			OgreBites::GameButton* btn = NULL;
			SdkTrayManager* trayMgr = NULL;
			Ogre::String prefix = "";
			int start = 0;
			switch (chapter)
			{
				case 1:
					trayMgr = mChapter1TrayMgr;
					prefix = "Play_Chapter_1_Level_";
					start = 1;
					break;
				case 2:
					trayMgr = mChapter2TrayMgr;
					prefix = "Play_Chapter_2_Level_";
					start = 21;
					break;
				case 3:
					trayMgr = mChapter3TrayMgr;
					prefix = "Play_Chapter_3_Level_";
					start = 41;
					break;
			}
			
			if (!trayMgr)
				return;
			
			if (!trayMgr->hasWidget(prefix + StringConverter::toString(start)))
				return;
			
			int maxUnlockedLevel = getMaxUnlockedLevel();
			
			// if (!jIsChaptersUnlocked() && !isAllChaptersUnlocked())
			// 	maxUnlockedLevel = MIN(maxUnlockedLevel, MAX_RAILS_FREE_LEVEL + 1);
			
			btn = (OgreBites::GameButton*)trayMgr->getWidget(prefix + StringConverter::toString(start));
			float levelBtnSize = btn->getOverlayElement()->getHeight();
			float captionPos1 = -(levelBtnSize * 0.3f);
			float captionPos2 = -(levelBtnSize / 3.0f);
			
			for (int i = start; i < start + 20; i++)
			{
				btn = (OgreBites::GameButton*)trayMgr->getWidget(prefix + StringConverter::toString(i));
				
				if (!btn)
					return;
				
				int stars = CPrefs::sharedPrefs()->starsOfLevel(i);
				
				if (i < maxUnlockedLevel && stars > 0)
				{
					int money = CPrefs::sharedPrefs()->recordMoneyOfLevel(i);
					Ogre::String matName = (stars == 1) ? "Rails/stars_1" : ((stars == 2) ? "Rails/stars_2" : "Rails/stars_3");
					btn->setMaterial(matName);
					btn->setHasPressedState(true);
					btn->setMaskMaterial("Rails/level_btn_score");
					btn->setCaption(StringConverter::toString(i));
					btn->setSecondCaption(StringConverter::toString(money));
					
					btn->setCaptionPosition(0, captionPos1);
				}
				else if (i == maxUnlockedLevel || (stars <= 0 && i <= maxUnlockedLevel))
				{
					btn->setMaterial("Rails/Current_level");
					btn->setHasPressedState(true);
					btn->setMaskMaterial("SdkTrays/Empty");
					btn->setCaption(StringConverter::toString(i));
					btn->setSecondCaption("");
					
					btn->setCaptionPosition(0, captionPos2);
				}
				else
				{
					btn->setMaterial("Rails/Level_locked");
					btn->setHasPressedState(false);
					btn->setMaskMaterial("SdkTrays/Empty");
					btn->setCaption("");
					btn->setSecondCaption("");
				}
			}
		}
		
		void setupChapter_1_TrayMgr()
		{
			if (!mChapter1TrayMgr)
			{
				mChapter1TrayMgr = new SdkTrayManager("Chapter_1_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mChapter1TrayMgr->hasWidget("Chapter_1_Map"))
			{
				mChapter1TrayMgr->setBackdropMaterial(getChaptersBackgroundMatName());
				Ogre::Overlay* bdLayer = mChapter1TrayMgr->getBackdropLayer();
				Ogre::Viewport* vp = mWindow->getViewport(0);
				float scaleX = 1.f / ((3.f * (float)vp->getActualWidth()) / (4.f* (float)vp->getActualHeight()));
				bdLayer->setScale(1.f, 1.f/scaleX);
				
				float f = (float)mWindow->getViewport(0)->getActualHeight() / 1536.f;
				f *= 1.2;
				OgreBites::GameButton* btn = NULL;
				CXPoint pt;
				
				float ff = (float)mWindow->getViewport(0)->getActualHeight() / 1536.f;
				OgreBites::DecorWidget* boxDW = mChapter1TrayMgr->createDecorWidget(TL_CENTER, "Chapter_1_Map", "Rails/Chapter_Map", ff);
				boxDW->getOverlayElement()->setMaterialName(getChapterMapMatName(1));
				
				//-----
				float chapter1_buttons[20][2] = {{284, 156}, {232, 252}, {198, 391}, {246, 480}, {288, 573}, {396, 555}, {508, 569}, {548, 673}, {694, 664}, {716, 535},
												 {620, 462}, {484, 423}, {352, 381}, {420, 263}, {494, 158}, {572, 254}, {606, 366}, {736, 376}, {776, 286}, {708, 183}};
				
				float levelBtnSize = 128 * f;
				//float sX = ((3.f * (float)vp->getActualWidth()) / (4.f* (float)vp->getActualHeight()));
				float lbScale = (float)mWindow->getViewport(0)->getActualHeight() / 768.f;
				float xOffset = (mWindow->getViewport(0)->getActualWidth() - lbScale * 1024.f) / 2.f;
				float wndHeight = mWindow->getViewport(0)->getActualHeight();
				float charHeight = levelBtnSize / 2.6f;
				float captionPos = -(levelBtnSize / 3.0f);
				
				CXPoint scoreSize = CXPoint(84 * ff, 56 * ff);
				float char2Height = charHeight * 0.6;
				float caption2PosX = (levelBtnSize / 3.0f);
				float caption2PosY = -(levelBtnSize / 2.0f);
				
				for (int i = 0; i < 20; i++)
				{
					int level = i + 1;
					btn = mChapter1TrayMgr->createGameButton(TL_NONE, "Play_Chapter_1_Level_" + StringConverter::toString(level), "Rails/Current_level", StringConverter::toString(level), true, levelBtnSize, levelBtnSize);
					btn->setTag(level);
					btn->getTextAreaElement()->setFontName(getFontNameForSize(charHeight, "BCaption"));
					btn->setCharHeight(charHeight);
					btn->setCaptionPosition(0, captionPos);
					btn->getOverlayElement()->setPosition((int)(xOffset + chapter1_buttons[i][0] * lbScale), (int)(wndHeight - chapter1_buttons[i][1] * lbScale));
					
					btn->setMaskOffsetAndSize(levelBtnSize - scoreSize.x * 0.8, - scoreSize.y * 0.19, scoreSize.x, scoreSize.y);
					btn->setFill(1);
					btn->setSecondCaptionCharHeight(char2Height);
					btn->setSecondCaptionPosition(caption2PosX, caption2PosY);
				}
				//-----
				
				OgreBites::Button* btn2 = NULL;
				pt = convertAbsCoordToScreen(CXPoint(0.5, 0.95));
				btn2 = mChapter1TrayMgr->createGameButton(TL_NONE, "Chapter1BackBtn", "Rails/BackButton", "", true, 218 * f, 126 * f);
				pt.x -= btn2->getOverlayElement()->getWidth() / 2;
				pt.y -= btn2->getOverlayElement()->getHeight() / 2;
				btn2->getOverlayElement()->setPosition((int)(pt.x), (int)(pt.y));
			}
			
			updateLevelsForChapter(1);
		}
		
		void setupChapter_2_TrayMgr()
		{
			if (!mChapter2TrayMgr)
			{
				mChapter2TrayMgr = new SdkTrayManager("Chapter_2_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mChapter2TrayMgr->hasWidget("Chapter_2_Map"))
			{
				mChapter2TrayMgr->setBackdropMaterial(getChaptersBackgroundMatName());
				Ogre::Overlay* bdLayer = mChapter2TrayMgr->getBackdropLayer();
				Ogre::Viewport* vp = mWindow->getViewport(0);
				float scaleX = 1.f / ((3.f * (float)vp->getActualWidth()) / (4.f* (float)vp->getActualHeight()));
				bdLayer->setScale(1.f, 1.f/scaleX);
				
				float f = (float)mWindow->getViewport(0)->getActualHeight() / 1536.f;
				f *= 1.2;
				OgreBites::GameButton* btn = NULL;
				CXPoint pt;
				
				float ff = (float)mWindow->getViewport(0)->getActualHeight() / 1536.f;
				OgreBites::DecorWidget* boxDW = mChapter2TrayMgr->createDecorWidget(TL_CENTER, "Chapter_2_Map", "Rails/Chapter_Map", ff);
				boxDW->getOverlayElement()->setMaterialName(getChapterMapMatName(2));
				
				//-----
				
				float chapter2_buttons[20][2] = {{190, 183}, {320, 232}, {458, 244}, {506, 344}, {382, 382}, {242, 350}, {152, 431}, {180, 559}, {270, 650}, {404, 669},
												 {476, 603}, {390, 498}, {484, 440}, {540, 550}, {650, 618}, {740, 537}, {656, 446}, {606, 337}, {710, 281}, {776, 183}};
				
				float levelBtnSize = 128 * f;
				//float sX = ((3.f * (float)vp->getActualWidth()) / (4.f* (float)vp->getActualHeight()));
				float lbScale = (float)mWindow->getViewport(0)->getActualHeight() / 768.f;
				float xOffset = (mWindow->getViewport(0)->getActualWidth() - lbScale * 1024.f) / 2.f;
				float wndHeight = mWindow->getViewport(0)->getActualHeight();
				float charHeight = levelBtnSize / 2.6f;
				float captionPos = -(levelBtnSize / 3.0f);
				
				CXPoint scoreSize = CXPoint(84 * ff, 56 * ff);
				float char2Height = charHeight * 0.6;
				float caption2PosX = (levelBtnSize / 3.0f);
				float caption2PosY = -(levelBtnSize / 2.0f);
				
				for (int i = 0; i < 20; i++)
				{
					int level = i + 21;
					btn = mChapter2TrayMgr->createGameButton(TL_NONE, "Play_Chapter_2_Level_" + StringConverter::toString(level), "Rails/Current_level", StringConverter::toString(level), true, levelBtnSize, levelBtnSize);
					btn->setTag(level);
					btn->getTextAreaElement()->setFontName(getFontNameForSize(charHeight, "BCaption"));
					btn->setCharHeight(charHeight);
					btn->setCaptionPosition(0, captionPos);
					btn->getOverlayElement()->setPosition((int)(xOffset + chapter2_buttons[i][0] * lbScale), (int)(wndHeight - chapter2_buttons[i][1] * lbScale));
					
					btn->setMaskOffsetAndSize(levelBtnSize - scoreSize.x * 0.8, - scoreSize.y * 0.19, scoreSize.x, scoreSize.y);
					btn->setFill(1);
					btn->setSecondCaptionCharHeight(char2Height);
					btn->setSecondCaptionPosition(caption2PosX, caption2PosY);
				}
				//-----
				
				OgreBites::Button* btn2 = NULL;
				pt = convertAbsCoordToScreen(CXPoint(0.5, 0.95));
				btn2 = mChapter2TrayMgr->createGameButton(TL_NONE, "Chapter2BackBtn", "Rails/BackButton", "", true, 218 * f, 126 * f);
				pt.x -= btn2->getOverlayElement()->getWidth() / 2;
				pt.y -= btn2->getOverlayElement()->getHeight() / 2;
				btn2->getOverlayElement()->setPosition((int)(pt.x), (int)(pt.y));
			}
			
			updateLevelsForChapter(2);
		}
		
		void setupChapter_3_TrayMgr()
		{
			if (!mChapter3TrayMgr)
			{
				mChapter3TrayMgr = new SdkTrayManager("Chapter_3_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mChapter3TrayMgr->hasWidget("Chapter_3_Map"))
			{
				mChapter3TrayMgr->setBackdropMaterial(getChaptersBackgroundMatName());
				Ogre::Overlay* bdLayer = mChapter3TrayMgr->getBackdropLayer();
				Ogre::Viewport* vp = mWindow->getViewport(0);
				float scaleX = 1.f / ((3.f * (float)vp->getActualWidth()) / (4.f* (float)vp->getActualHeight()));
				bdLayer->setScale(1.f, 1.f/scaleX);
				
				float f = (float)mWindow->getViewport(0)->getActualHeight() / 1536.f;
				f *= 1.2;
				OgreBites::GameButton* btn = NULL;
				CXPoint pt;
				
				float ff = (float)mWindow->getViewport(0)->getActualHeight() / 1536.f;
				OgreBites::DecorWidget* boxDW = mChapter3TrayMgr->createDecorWidget(TL_CENTER, "Chapter_3_Map", "Rails/Chapter_Map", ff);
				boxDW->getOverlayElement()->setMaterialName(getChapterMapMatName(3));
				
				//-----
				
				float chapter3_buttons[20][2] = {{188, 183}, {328, 213}, {462, 213}, {522, 308}, {408, 303}, {308, 350}, {188, 425}, {132, 529}, {172, 631}, {282, 679},
												 {420, 669}, {532, 631}, {642, 669}, {776, 654}, {790, 564}, {716, 504}, {592, 439}, {674, 355}, {796, 308}, {776, 185}};
				
				float levelBtnSize = 128 * f;
				//float sX = ((3.f * (float)vp->getActualWidth()) / (4.f* (float)vp->getActualHeight()));
				float lbScale = (float)mWindow->getViewport(0)->getActualHeight() / 768.f;
				float xOffset = (mWindow->getViewport(0)->getActualWidth() - lbScale * 1024.f) / 2.f;
				float wndHeight = mWindow->getViewport(0)->getActualHeight();
				float charHeight = levelBtnSize / 2.6f;
				float captionPos = -(levelBtnSize / 3.0f);
				
				CXPoint scoreSize = CXPoint(84 * ff, 56 * ff);
				float char2Height = charHeight * 0.6;
				float caption2PosX = (levelBtnSize / 3.0f);
				float caption2PosY = -(levelBtnSize / 2.0f);
				
				for (int i = 0; i < 20; i++)
				{
					int level = i + 41;
					btn = mChapter3TrayMgr->createGameButton(TL_NONE, "Play_Chapter_3_Level_" + StringConverter::toString(level), "Rails/Current_level", StringConverter::toString(level), true, levelBtnSize, levelBtnSize);
					btn->setTag(level);
					btn->getTextAreaElement()->setFontName(getFontNameForSize(charHeight, "BCaption"));
					btn->setCharHeight(charHeight);
					btn->setCaptionPosition(0, captionPos);
					btn->getOverlayElement()->setPosition((int)(xOffset + chapter3_buttons[i][0] * lbScale), (int)(wndHeight - chapter3_buttons[i][1] * lbScale));
					
					btn->setMaskOffsetAndSize(levelBtnSize - scoreSize.x * 0.8, - scoreSize.y * 0.19, scoreSize.x, scoreSize.y);
					btn->setFill(1);
					btn->setSecondCaptionCharHeight(char2Height);
					btn->setSecondCaptionPosition(caption2PosX, caption2PosY);
				}
				//-----
				
				OgreBites::Button* btn2 = NULL;
				pt = convertAbsCoordToScreen(CXPoint(0.5, 0.95));
				btn2 = mChapter3TrayMgr->createGameButton(TL_NONE, "Chapter3BackBtn", "Rails/BackButton", "", true, 218 * f, 126 * f);
				pt.x -= btn2->getOverlayElement()->getWidth() / 2;
				pt.y -= btn2->getOverlayElement()->getHeight() / 2;
				btn2->getOverlayElement()->setPosition((int)(pt.x), (int)(pt.y));
			}
			
			updateLevelsForChapter(3);
		}
		
		Ogre::String scoreForCategoryAtIndex(int idx)
		{
			switch (idx) {
				case 0:
					return "$" + StringConverter::toString((long unsigned int)CPrefs::sharedPrefs()->mainScore());
					break;
			
				case 1:
					return "$" + StringConverter::toString((long unsigned int)CPrefs::sharedPrefs()->chapter1Score());
					break;
			
				case 2:
					return "$" + StringConverter::toString((long unsigned int)CPrefs::sharedPrefs()->chapter2Score());
					break;
			
				case 3:
					return "$" + StringConverter::toString((long unsigned int)CPrefs::sharedPrefs()->chapter3Score());
					break;
			
				case 4:
		            {
		                Ogre::String appendix((CPrefs::sharedPrefs()->countArrivedScore() == 1) ? "train" : "trains");
		                appendix = jGetLocalizedString(appendix.c_str());
		                return StringConverter::toString((long unsigned int)CPrefs::sharedPrefs()->countArrivedScore()) + " " + appendix;
		            }
					break;
			
				case 5:
					return "$" + StringConverter::toString((long unsigned int)CPrefs::sharedPrefs()->grossMoneyScore());
					break;
			
				case 6:
					{
		                Ogre::String appendix((CPrefs::sharedPrefs()->countArrivedScore() == 1) ? "train" : "trains");
		                appendix = jGetLocalizedString(appendix.c_str());
		                return StringConverter::toString((long unsigned int)CPrefs::sharedPrefs()->countCrashedScore()) + " " + appendix;
					}
					break;
			
				default:
					return "Unknown";
					break;
			}
		}

		Ogre::String nameForCategoryAtIndex(int idx)
		{
		    Ogre::String str;
    
			switch (idx) {
				case 0: str =  "Monetary_Board";   break;
				case 1: str =  "Chapter_1";        break;
				case 2: str =  "Chapter_2";        break;
				case 3: str =  "Chapter_3";        break;
				case 4: str =  "Stationed_Trains"; break;
				case 5: str =  "Gross_Earnings";   break;
				case 6: str =  "Public_Enemy_Crashes"; break;
				default:str =  "Unknown";          break;
			}
    
		    str = jGetLocalizedString(str.c_str());
    
		    return str;
		}
		
		Ogre::String matNameForCategoryAtIndex(int idx)
		{
		    Ogre::String str;
    
			switch (idx) {
				case 0: str =  "HighScores/money";    break;
				case 1: str =  "HighScores/chapter1"; break;
				case 2: str =  "HighScores/chapter2"; break;
				case 3: str =  "HighScores/chapter3"; break;
				case 4: str =  "HighScores/trains";   break;
				case 5: str =  "HighScores/gross";    break;
				case 6: str =  "HighScores/crashes";  break;
				default:str =  "SdkTrays/Empty";      break;
			}
    
		    return str;
		}
		
		// bool needsUseLowResTextures()
		// {
		// 	Ogre::Viewport* vp = mWindow->getViewport(0);
		// 	if (vp)
		// 		return mWindow->getViewport(0)->getActualHeight() < 1024;
		// 	else
		// 		return false;
		// }
		
		void setupHighScoresTrayMgr()
		{
			if (!mHighScoresTrayMgr)
			{
				mHighScoresTrayMgr = new SdkTrayManager("MMHighScores_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mHighScoresTrayMgr->hasWidget("HighScoresBackPanel"))
			{
				mHighScoresTrayMgr->setBackdropMaterial("SdkTrays/Shade");
				mHighScoresTrayMgr->getBackdropLayer()->setZOrder(410);
				mHighScoresTrayMgr->getTraysLayer()->setZOrder(411);
				
				float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
				Ogre::Viewport* vp = mWindow->getViewport(0);
				f *= (jNeedsUseLowResOverlays()) ? 1.f : 0.75f;
				
				OgreBites::DecorWidget* dlgBack = mHighScoresTrayMgr->createDecorWidget(TL_CENTER, "HighScoresBackPanel", "Rails/BlackOpaquePanel", f);
				Ogre::BorderPanelOverlayElement* panelEl = (Ogre::BorderPanelOverlayElement*)dlgBack->getOverlayElement();
				if (jNeedsUseLowResTextures())
					panelEl->setBorderSize(20, 20, 20, 20);
				dlgBack->getOverlayElement()->setDimensions(700 * f, 900 * f);
				mHighScoresTrayMgr->adjustTrays();
				
				OgreBites::Label* lbl = NULL;
				
				lbl = mHighScoresTrayMgr->createLabel(TL_NONE, "HighScoresCaption", jGetLocalizedString("High_Scores"), dlgBack->getOverlayElement()->getWidth() * 0.8, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(midCaptionSize(), "BCaption"));
	  			lbl->setCharHeight(midCaptionSize());
	  			lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
	  			lbl->getOverlayElement()->setPosition(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 2.0,
													(vp->getActualHeight() - dlgBack->getOverlayElement()->getHeight()) / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.01f);
													
				
				mHighScoresBox = (Ogre::BorderPanelOverlayElement*)dlgBack->getOverlayElement();
			
				float elWidth = dlgBack->getOverlayElement()->getWidth() * 0.8f;
				float elHeight = dlgBack->getOverlayElement()->getWidth() / 8.f;
				int hsCount = 7;
				int size = elHeight;
				int linesCount = 0;
				float textWidth = elWidth - size * 1.1;//e->getWidth() * 0.8;
				float charHeight = elHeight * 0.42;
			
				for (int idx = 0; idx < hsCount; idx++)
				{
					Ogre::OverlayElement* e = (Ogre::OverlayElement*)Ogre::OverlayManager::getSingleton().createOverlayElementFromTemplate("Rails/AchievementItem", "", "HighScoresItem_" + Ogre::StringConverter::toString(idx));
					e->setDimensions(elWidth, elHeight);
					
					Ogre::OverlayElement* icon = (Ogre::OverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementIcon");
					icon->setMaterialName(matNameForCategoryAtIndex(idx));
					icon->setDimensions(size, size);
					
					Ogre::TextAreaOverlayElement* title = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementTitle");
					title->setAlignment(Ogre::TextAreaOverlayElement::Left);
					title->setHorizontalAlignment(Ogre::GHA_LEFT);
					title->setFontName(getFontNameForSize(charHeight, "BCaption"));
					title->setCharHeight(charHeight);
					title->setWidth(textWidth);
					title->setHeight(charHeight);
					title->setCaption(nameForCategoryAtIndex(idx));
					title->setLeft(icon->getWidth() * 1.3);
					title->setTop( - size * 0.3 );
									
					Ogre::TextAreaOverlayElement* desc = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementDesc");
					desc->setAlignment(Ogre::TextAreaOverlayElement::Left);
					desc->setHorizontalAlignment(Ogre::GHA_LEFT);
					desc->setFontName(getFontNameForSize(charHeight, "Caption"));
					desc->setCharHeight(charHeight);
					desc->setWidth(textWidth);
					desc->setCaption(scoreForCategoryAtIndex(idx));
					desc->setHeight(charHeight);
					desc->setLeft(icon->getWidth() * 1.3);
					desc->setTop( size * 0.1 );
									
					e->setPosition( - e->getWidth() * 0.5, (e->getHeight() * 1.2) * (idx + 1));
					
								
					mHighScoresBox->addChild(e);
					mHighScoresItems.push_back(e);
				}
													
			}
			else
			{
				int hsCount = 7;
				int linesCount = 0;
				
				for (int idx = 0; idx < hsCount; idx++)
				{
					if (idx > mHighScoresItems.size())
						break;
					
					Ogre::OverlayElement* e = mHighScoresItems[idx];
					
					Ogre::TextAreaOverlayElement* desc = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementDesc");
					desc->setCaption(scoreForCategoryAtIndex(idx));
				}
				
			}
		}
		
		float smallCaptionSize()
		{
			float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
			f *= (jNeedsUseLowResOverlays()) ? 1.f : 0.75f;
			
			return 40 * f;
		}
		
		float midCaptionSize()
		{
			float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
			f *= (jNeedsUseLowResOverlays()) ? 1.f : 0.75f;
			
			return 60 * f;
		}
		
		
		void setupUnlockChaptersOfferTrayMgr()
		{
			if (!mUnlockChaptersOfferTrayMgr)
			{
				mUnlockChaptersOfferTrayMgr = new SdkTrayManager("UnlockChaptersOffer_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mUnlockChaptersOfferTrayMgr->hasWidget("UnlockChaptersOfferBackPanel"))
			{
				mUnlockChaptersOfferTrayMgr->setBackdropMaterial("SdkTrays/Shade");
				mUnlockChaptersOfferTrayMgr->getBackdropLayer()->setZOrder(410);
				mUnlockChaptersOfferTrayMgr->getTraysLayer()->setZOrder(411);
				
				Ogre::Viewport* vp = mWindow->getViewport(0);
				float f = (float)vp->getActualHeight() / 1600.f;
				f *= (jNeedsUseLowResOverlays()) ? 1.f : 0.75f;
				
				/*
				OgreBites::DecorWidget* dlgBack = mUnlockChaptersOfferTrayMgr->createDecorWidget(TL_CENTER, "UnlockChaptersOfferBackPanel", "Rails/BlackOpaquePanel", f);
				Ogre::BorderPanelOverlayElement* panelEl = (Ogre::BorderPanelOverlayElement*)dlgBack->getOverlayElement();
				if (jNeedsUseLowResOverlays())
					panelEl->setBorderSize(20, 20, 20, 20);
				dlgBack->getOverlayElement()->setDimensions(800 * f, 600 * f);
				*/
				OgreBites::DecorWidget* dlgBack = mUnlockChaptersOfferTrayMgr->createDecorWidget(TL_CENTER, "UnlockChaptersOfferBackPanel", "Rails/BuyDialogBackPanel", f);
				dlgBack->getOverlayElement()->setMaterialName("Rails/BuyDialogBack");
				mUnlockChaptersOfferTrayMgr->adjustTrays();
				
				OgreBites::Label* lbl = NULL;
				float charHeight = dlgBack->getOverlayElement()->getHeight() * 0.06;
				float lblOffset = (vp->getActualHeight() - dlgBack->getOverlayElement()->getHeight()) / 2.f;
				
				lbl = mUnlockChaptersOfferTrayMgr->createLabel(TL_NONE, "UnlockChaptersOffer_Caption", jGetLocalizedString("billing_header"), dlgBack->getOverlayElement()->getWidth() * 0.9, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(charHeight, "BCaption"));
	  			lbl->setCharHeight(charHeight);
	  			lbl->setColour(Ogre::ColourValue(0, 0, 0, 1));
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
				lbl->getTextAreaElement()->setTop( - lbl->getOverlayElement()->getHeight() / 2.f);
	  			lbl->getOverlayElement()->setLeft(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 2.0);
				lblOffset += dlgBack->getOverlayElement()->getHeight() * 0.03;
				lbl->getOverlayElement()->setTop(lblOffset);
				lblOffset += lbl->getCharHeight() / 2.f + 15 * f;
				
				lbl = mUnlockChaptersOfferTrayMgr->createLabel(TL_NONE, "UnlockChaptersOffer_Description", "", dlgBack->getOverlayElement()->getWidth() * 0.9, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(charHeight * 0.72, "Caption"));
				lbl->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
				lbl->getTextAreaElement()->setHorizontalAlignment(Ogre::GHA_LEFT);
	  			lbl->setCharHeight(charHeight * 0.72);
	  			lbl->setColour(Ogre::ColourValue(0, 0, 0, 1));
				lbl->setWrappedText(jGetLocalizedString("billing_text"));
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
				lbl->getTextAreaElement()->setTop( - lbl->getOverlayElement()->getHeight() / 2.f);
				lbl->getOverlayElement()->setLeft(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 2.0);
				int cnt = lbl->getLinesCount();
				float offset = (lbl->getCharHeight() * cnt) / 2.f;
				lbl->getOverlayElement()->setTop(lblOffset + dlgBack->getOverlayElement()->getHeight() * 0.11 - offset);
				
													
				//if (!jIsChaptersUnlocked())
				{
					OgreBites::Button* btn = NULL;
					int chSize = 56 * f;
					btn = mUnlockChaptersOfferTrayMgr->createGameButton(TL_NONE, "BuyDialogButtonBack", "Rails/BuyDialogButtonBack", "", true, 360 * f, 148 * f);
					btn->setCharHeight(chSize);
					btn->setCaptionPosition(0, - btn->getOverlayElement()->getHeight() * 0.3);
					btn->getTextAreaElement()->setFontName(getFontNameForSize(chSize, "BICaption"));
	  				btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f + dlgBack->getOverlayElement()->getWidth() / 2.f - btn->getOverlayElement()->getWidth() * 2.7f);
	  				btn->getOverlayElement()->setTop(vp->getActualHeight() / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.5f  - btn->getOverlayElement()->getHeight() * 2.f);
					
					btn = mUnlockChaptersOfferTrayMgr->createGameButton(TL_NONE, "BuyDialogButtonBuy", "Rails/BuyDialogButtonBuy", "", true, 360 * f, 148 * f);
					btn->setCharHeight(chSize);
					btn->setCaptionPosition(0, - btn->getOverlayElement()->getHeight() * 0.3);
					btn->getTextAreaElement()->setFontName(getFontNameForSize(chSize, "BICaption"));
	  				btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f + dlgBack->getOverlayElement()->getWidth() / 2.f - btn->getOverlayElement()->getWidth() * 1.5f);
	  				btn->getOverlayElement()->setTop(vp->getActualHeight() / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.5f  - btn->getOverlayElement()->getHeight() * 2.f);
				}
				
			}
		}
		
		void setupDebugTrayMgr()
		{
			if (!mDebugTrayMgr)
			{
				mDebugTrayMgr = new SdkTrayManager("DebugDialog_TrayMgr", mWindow, mInputContext, this);
			}
#ifdef ENABLE_DEBUG_DIALOGS
			if (!mDebugTrayMgr->hasWidget("DebugBackPanel"))
			{
				mDebugTrayMgr->setBackdropMaterial("SdkTrays/Shade");
				mDebugTrayMgr->getBackdropLayer()->setZOrder(410);
				mDebugTrayMgr->getTraysLayer()->setZOrder(411);
				
				float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
				Ogre::Viewport* vp = mWindow->getViewport(0);
				f *= (jNeedsUseLowResOverlays()) ? 1.f : 0.75f;
				
				OgreBites::DecorWidget* dlgBack = mDebugTrayMgr->createDecorWidget(TL_CENTER, "DebugBackPanel", "Rails/BlackOpaquePanel", f);
				Ogre::BorderPanelOverlayElement* panelEl = (Ogre::BorderPanelOverlayElement*)dlgBack->getOverlayElement();
				if (jNeedsUseLowResTextures())
					panelEl->setBorderSize(20, 20, 20, 20);
#ifdef GOOGLE_PLUS_ENABLED
				dlgBack->getOverlayElement()->setDimensions(1300 * f, 870 * f);
#else
				dlgBack->getOverlayElement()->setDimensions(1300 * f, 820 * f);
#endif
				
				OgreBites::GameButton* btn = NULL;
				Ogre::BorderPanelOverlayElement* btnBPOE = NULL;
				float captWidth = 0;
				
				float ff = f * 1.5;
				float topOffset = vp->getActualHeight() / 2.f - dlgBack->getOverlayElement()->getHeight() * 0.5f;
				btn = mDebugTrayMgr->createGameButton(TL_NONE, "MM_ShowRandomAchievements", "Rails/check_off", "Show Random Achievements in Complete Level View", false, 58 * ff, 62 * ff);
				btn->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
				btn->getTextAreaElement()->setHorizontalAlignment(Ogre::GHA_LEFT);
				btn->setCharHeight(midCaptionSize() * 0.85);
				btn->setCaptionPosition(btn->getOverlayElement()->getWidth(), - btn->getTextAreaElement()->getCharHeight() / 2.f);
	  			btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - dlgBack->getOverlayElement()->getWidth() * 0.46f);
				topOffset += btn->getOverlayElement()->getHeight() * 0.4;
				btn->getOverlayElement()->setTop(topOffset);
				
				btn = mDebugTrayMgr->createGameButton(TL_NONE, "Chapters_UnlockCheck", "Rails/check_off", "Unlock Chapters", false, 58 * ff, 62 * ff);
				btn->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
				btn->getTextAreaElement()->setHorizontalAlignment(Ogre::GHA_LEFT);
				btn->setCharHeight(midCaptionSize() * 0.85);
				btn->setCaptionPosition(btn->getOverlayElement()->getWidth(), - btn->getTextAreaElement()->getCharHeight() / 2.f);
	  			btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - dlgBack->getOverlayElement()->getWidth() * 0.46f);
				topOffset += btn->getOverlayElement()->getHeight() * 1.1;
				btn->getOverlayElement()->setTop(topOffset);
				
				
				btn = mDebugTrayMgr->createGameButton(TL_NONE, "Chapters_IgnoreLBA", "Rails/check_off", "Ignore locking by achievements", false, 58 * ff, 62 * ff);
				btn->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
				btn->getTextAreaElement()->setHorizontalAlignment(Ogre::GHA_LEFT);
				btn->setCharHeight(midCaptionSize() * 0.85);
				btn->setCaptionPosition(btn->getOverlayElement()->getWidth(), - btn->getTextAreaElement()->getCharHeight() / 2.f);
	  			btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - dlgBack->getOverlayElement()->getWidth() * 0.46f);
				topOffset += btn->getOverlayElement()->getHeight() * 1.1;
				btn->getOverlayElement()->setTop(topOffset);
				
  				btn = mDebugTrayMgr->createGameButton(TL_NONE, "RegisterAndRate_EraseButton", "Rails/EraseButton", "Erase Register and Rate Dialogs Info", true, 64 * ff, 100 * f);
				btnBPOE = (Ogre::BorderPanelOverlayElement*)btn->getOverlayElement();
				btn->setBorderMaterial("Rails/EraseButton");
				btn->setCharHeight(midCaptionSize() * 0.85);
				btnBPOE->setBorderSize(16.0 * f, 16.0 * f, 0, 0);
				btnBPOE->setUV(0.25, 0, 0.75, 1.0);
				btnBPOE->setLeftBorderUV(0.0, 0, 0.25, 1.0);
				btnBPOE->setRightBorderUV(0.75, 0, 1.0, 1.0);
				captWidth = Widget::getCaptionWidth(btn->getCaption(), btn->getTextAreaElement()) * 1.5;
				btn->getOverlayElement()->setWidth(captWidth);
				btn->getTextAreaElement()->setTop(- btn->getTextAreaElement()->getCharHeight() * 0.5);
	  			btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() / 2.0);
				topOffset += btn->getOverlayElement()->getHeight() * 1.2;
				btn->getOverlayElement()->setTop(topOffset);
				
  				btn = mDebugTrayMgr->createGameButton(TL_NONE, "Preferences_EraseButton", "Rails/EraseButton", "Erase Preferences Info but keep Cloud data.", true, 64 * ff, 100 * f);
				btnBPOE = (Ogre::BorderPanelOverlayElement*)btn->getOverlayElement();
				btn->setBorderMaterial("Rails/EraseButton");
				btn->setCharHeight(midCaptionSize() * 0.85);
				btnBPOE->setBorderSize(16.0 * f, 16.0 * f, 0, 0);
				btnBPOE->setUV(0.25, 0, 0.75, 1.0);
				btnBPOE->setLeftBorderUV(0.0, 0, 0.25, 1.0);
				btnBPOE->setRightBorderUV(0.75, 0, 1.0, 1.0);
				btn->getOverlayElement()->setWidth(captWidth);
				btn->getTextAreaElement()->setTop(- btn->getTextAreaElement()->getCharHeight() * 0.5);
	  			btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() / 2.0);
				topOffset += btn->getOverlayElement()->getHeight() * 1.2;
				btn->getOverlayElement()->setTop(topOffset);
				
  				btn = mDebugTrayMgr->createGameButton(TL_NONE, "Snapshot_EraseButton", "Rails/EraseButton", "Erase Snapshot", true, 64 * ff, 100 * f);
				btnBPOE = (Ogre::BorderPanelOverlayElement*)btn->getOverlayElement();
				btn->setBorderMaterial("Rails/EraseButton");
				btn->setCharHeight(midCaptionSize() * 0.85);
				btnBPOE->setBorderSize(16.0 * f, 16.0 * f, 0, 0);
				btnBPOE->setUV(0.25, 0, 0.75, 1.0);
				btnBPOE->setLeftBorderUV(0.0, 0, 0.25, 1.0);
				btnBPOE->setRightBorderUV(0.75, 0, 1.0, 1.0);
				btn->getOverlayElement()->setWidth(captWidth);
				btn->getTextAreaElement()->setTop(- btn->getTextAreaElement()->getCharHeight() * 0.5);
	  			btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() / 2.0);
				topOffset += btn->getOverlayElement()->getHeight() * 1.2;
				btn->getOverlayElement()->setTop(topOffset);
#ifdef GOOGLE_PLUS_ENABLED
  				btn = mDebugTrayMgr->createGameButton(TL_NONE, "CloudData_EraseButton", "Rails/EraseButton", "Erase All Cloud Data", true, 64 * ff, 100 * f);
				btnBPOE = (Ogre::BorderPanelOverlayElement*)btn->getOverlayElement();
				btn->setBorderMaterial("Rails/EraseButton");
				btn->setCharHeight(midCaptionSize() * 0.85);
				btnBPOE->setBorderSize(16.0 * f, 16.0 * f, 0, 0);
				btnBPOE->setUV(0.25, 0, 0.75, 1.0);
				btnBPOE->setLeftBorderUV(0.0, 0, 0.25, 1.0);
				btnBPOE->setRightBorderUV(0.75, 0, 1.0, 1.0);
				btn->getOverlayElement()->setWidth(captWidth);
				btn->getTextAreaElement()->setTop(- btn->getTextAreaElement()->getCharHeight() * 0.5);
	  			btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() / 2.0);
				topOffset += btn->getOverlayElement()->getHeight() * 1.2;
				btn->getOverlayElement()->setTop(topOffset);
#endif
				mDebugTrayMgr->adjustTrays();
			}
#endif
		}
		
		void setupOptionsTrayMgr()
		{
			if (!mOptionsTrayMgr)
			{
				mOptionsTrayMgr = new SdkTrayManager("MMOptions_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mOptionsTrayMgr->hasWidget("OptionsBackPanel"))
			{
				mOptionsTrayMgr->setBackdropMaterial("SdkTrays/Shade");
				mOptionsTrayMgr->getBackdropLayer()->setZOrder(410);
				mOptionsTrayMgr->getTraysLayer()->setZOrder(411);
				
				float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
				Ogre::Viewport* vp = mWindow->getViewport(0);
				f *= (jNeedsUseLowResOverlays()) ? 1.f : 0.75f;
				
				OgreBites::DecorWidget* dlgBack = mOptionsTrayMgr->createDecorWidget(TL_CENTER, "OptionsBackPanel", "Rails/BlackOpaquePanel", f);
				Ogre::BorderPanelOverlayElement* panelEl = (Ogre::BorderPanelOverlayElement*)dlgBack->getOverlayElement();
				if (jNeedsUseLowResTextures())
					panelEl->setBorderSize(20, 20, 20, 20);
#ifdef GOOGLE_PLUS_ENABLED
				dlgBack->getOverlayElement()->setDimensions(850 * f, 700 * f);
#else
				dlgBack->getOverlayElement()->setDimensions(800 * f, 600 * f);
#endif
				mOptionsTrayMgr->adjustTrays();
				
				OgreBites::Label* lbl = NULL;
				float charHeight = midCaptionSize() * 0.7;
				
				lbl = mOptionsTrayMgr->createLabel(TL_NONE, "Options_Caption", jGetLocalizedString("Options"), dlgBack->getOverlayElement()->getWidth() * 0.8, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(midCaptionSize(), "BCaption"));
	  			lbl->setCharHeight(midCaptionSize());
	  			lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
	  			lbl->getOverlayElement()->setLeft(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 2.0);
#ifdef GOOGLE_PLUS_ENABLED
				lbl->getOverlayElement()->setTop(vp->getActualHeight() / 2.f - dlgBack->getOverlayElement()->getHeight() * 0.34f - lbl->getOverlayElement()->getHeight() * 1.4);
#else
				lbl->getOverlayElement()->setTop(vp->getActualHeight() / 2.f - dlgBack->getOverlayElement()->getHeight() * 0.28f - lbl->getOverlayElement()->getHeight() * 1.4);
#endif
				
				lbl = mOptionsTrayMgr->createLabel(TL_NONE, "Options_SoundCaption", jGetLocalizedString("Sound"), dlgBack->getOverlayElement()->getWidth() * 0.5, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(charHeight, "BCaption"));
				lbl->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
				lbl->getTextAreaElement()->setHorizontalAlignment(Ogre::GHA_LEFT);
	  			lbl->setCharHeight(charHeight);
	  			lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
	  			lbl->getOverlayElement()->setLeft(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() * 0.1f);
#ifdef GOOGLE_PLUS_ENABLED
				lbl->getOverlayElement()->setTop(vp->getActualHeight() / 2.f - dlgBack->getOverlayElement()->getHeight() * 0.22f - lbl->getOverlayElement()->getHeight());
#else
				lbl->getOverlayElement()->setTop(vp->getActualHeight() / 2.f - dlgBack->getOverlayElement()->getHeight() * 0.12f - lbl->getOverlayElement()->getHeight());
#endif
													
				lbl = mOptionsTrayMgr->createLabel(TL_NONE, "Options_MusicCaption", jGetLocalizedString("Music"), dlgBack->getOverlayElement()->getWidth() * 0.5, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(charHeight, "BCaption"));
				lbl->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
				lbl->getTextAreaElement()->setHorizontalAlignment(Ogre::GHA_LEFT);
	  			lbl->setCharHeight(charHeight);
	  			lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
	  			lbl->getOverlayElement()->setLeft(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() * 0.1f);
#ifdef GOOGLE_PLUS_ENABLED
				lbl->getOverlayElement()->setTop(vp->getActualHeight() / 2.f - dlgBack->getOverlayElement()->getHeight() * 0.04f - lbl->getOverlayElement()->getHeight());
#else
				lbl->getOverlayElement()->setTop(vp->getActualHeight() / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.06f - lbl->getOverlayElement()->getHeight());
#endif
				
#ifdef GOOGLE_PLUS_ENABLED
  				lbl = mOptionsTrayMgr->createLabel(TL_NONE, "Options_GPlusCaption", jGetLocalizedString("sign_in"), dlgBack->getOverlayElement()->getWidth() * 0.5, true);
  	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(charHeight, "BCaption"));
  				lbl->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
  				lbl->getTextAreaElement()->setHorizontalAlignment(Ogre::GHA_LEFT);
  	  			lbl->setCharHeight(charHeight);
  	  			lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
  	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
  	  			lbl->getOverlayElement()->setLeft(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() * 0.1f);
  				lbl->getOverlayElement()->setTop(vp->getActualHeight() / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.14f - lbl->getOverlayElement()->getHeight());
#endif
													
				OgreBites::GameButton* btn = NULL;
				float ff = f * 1.5;
				btn = mOptionsTrayMgr->createGameButton(TL_NONE, "Options_SoundButton", "Rails/check_on", "", false, 58 * ff, 62 * ff);
				btn->setMaterial(CPrefs::sharedPrefs()->isSoundOn() ? "Rails/check_on" : "Rails/check_off");
				btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() * 1.8f);
#ifdef GOOGLE_PLUS_ENABLED
				btn->getOverlayElement()->setTop(vp->getActualHeight() / 2.f - dlgBack->getOverlayElement()->getHeight() * 0.22f  - btn->getOverlayElement()->getHeight() / 2.f);
#else
				btn->getOverlayElement()->setTop(vp->getActualHeight() / 2.f - dlgBack->getOverlayElement()->getHeight() * 0.12f  - btn->getOverlayElement()->getHeight() / 2.f);
#endif
				
  				btn = mOptionsTrayMgr->createGameButton(TL_NONE, "Options_MusicButton", "Rails/check_on", "", false, 58 * ff, 62 * ff);
  				btn->setMaterial(CPrefs::sharedPrefs()->isMusicOn() ? "Rails/check_on" : "Rails/check_off");
  				btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() * 1.8f);
#ifdef GOOGLE_PLUS_ENABLED
  				btn->getOverlayElement()->setTop(vp->getActualHeight() / 2.f - dlgBack->getOverlayElement()->getHeight() * 0.04f  - btn->getOverlayElement()->getHeight() / 2.f);
#else
  				btn->getOverlayElement()->setTop(vp->getActualHeight() / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.06f  - btn->getOverlayElement()->getHeight() / 2.f);
#endif
				
#ifdef GOOGLE_PLUS_ENABLED
  				btn = mOptionsTrayMgr->createGameButton(TL_NONE, "Options_GPlus", "Rails/GPlus_Unsigned", "", false, 50 * ff, 50 * ff);
  				btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() * 2.0f);
  				btn->getOverlayElement()->setTop(vp->getActualHeight() / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.14f  - btn->getOverlayElement()->getHeight() / 2.f);
#endif

  				btn = mOptionsTrayMgr->createGameButton(TL_NONE, "Options_EraseButton", "Rails/EraseButton", jGetLocalizedString("Erase_game_progress"), true, 64 * f, 100 * f);
				mEraseButton = btn;
				Ogre::BorderPanelOverlayElement* btnBPOE = (Ogre::BorderPanelOverlayElement*)btn->getOverlayElement();
				btn->setBorderMaterial("Rails/EraseButton");
				btn->setCharHeight(charHeight);
				btnBPOE->setBorderSize(16.0 * f, 16.0 * f, 0, 0);
				btnBPOE->setUV(0.25, 0, 0.75, 1.0);
				btnBPOE->setLeftBorderUV(0.0, 0, 0.25, 1.0);
				btnBPOE->setRightBorderUV(0.75, 0, 1.0, 1.0);
				float captW = Widget::getCaptionWidth(btn->getCaption(), btn->getTextAreaElement());
				btn->getOverlayElement()->setWidth(captW * 1.2);
  				btn->getOverlayElement()->setLeft(vp->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() * 0.5f);
#ifdef GOOGLE_PLUS_ENABLED
  				btn->getOverlayElement()->setTop(vp->getActualHeight() / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.32f  - btn->getOverlayElement()->getHeight() / 2.f);
#else
  				btn->getOverlayElement()->setTop(vp->getActualHeight() / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.26f  - btn->getOverlayElement()->getHeight() / 2.f);
#endif
				btn->getTextAreaElement()->setTop(- btn->getTextAreaElement()->getCharHeight() * 0.5);
				
				lbl = mOptionsTrayMgr->createLabel(TL_NONE, "Options_EraseProgressLabel", "", dlgBack->getOverlayElement()->getWidth() * 0.85, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(smallCaptionSize() * 0.8f, "Caption"));
				lbl->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Center);
				lbl->getTextAreaElement()->setHorizontalAlignment(Ogre::GHA_CENTER);
	  			lbl->setCharHeight(smallCaptionSize() * 0.8f);
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight() * 2.f);
	  			lbl->getOverlayElement()->setLeft(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() * 0.5f);
				lbl->getTextAreaElement()->setTop( - lbl->getOverlayElement()->getHeight() / 2.f);
			}
			else
			{
				OgreBites::GameButton* btn = (OgreBites::GameButton*)mOptionsTrayMgr->getWidget("Options_SoundButton");
				btn->setMaterial(CPrefs::sharedPrefs()->isSoundOn() ? "Rails/check_on" : "Rails/check_off");
				
				btn = (OgreBites::GameButton*)mOptionsTrayMgr->getWidget("Options_MusicButton");
				btn->setMaterial(CPrefs::sharedPrefs()->isMusicOn() ? "Rails/check_on" : "Rails/check_off");
				
				setEraseProgressLabelString("", Ogre::ColourValue(1, 1, 1, 1));
			}
			
#ifdef GOOGLE_PLUS_ENABLED
			updateWidgetStates(true);
#endif
		}
		
		void setEraseProgressLabelString(std::string str, Ogre::ColourValue color)
		{
			if (mOptionsTrayMgr->hasWidget(TL_NONE, "Options_EraseProgressLabel"))
			{
				Ogre::Viewport* vp = mWindow->getViewport(0);
				OgreBites::DecorWidget* dlgBack = (OgreBites::DecorWidget*)mOptionsTrayMgr->getWidget(TL_CENTER, "OptionsBackPanel");
				OgreBites::Label* lbl = (OgreBites::Label*)mOptionsTrayMgr->getWidget(TL_NONE, "Options_EraseProgressLabel");
				lbl->setColour(color);
				lbl->setWrappedText(str);
				int cnt = lbl->getLinesCount();
				float offset = lbl->getCharHeight() * cnt / 2.f;
#ifdef GOOGLE_PLUS_ENABLED
				lbl->getOverlayElement()->setTop(vp->getActualHeight() / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.43f - offset);
#else
				lbl->getOverlayElement()->setTop(vp->getActualHeight() / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.4f - offset);
#endif
			}
		}
		
		void setupCreditsTrayMgr()
		{
			if (!mCreditsTrayMgr)
			{
				mCreditsTrayMgr = new SdkTrayManager("MMCredits_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mCreditsTrayMgr->hasWidget("CreditsBackPanel"))
			{
				mCreditsTrayMgr->setBackdropMaterial("SdkTrays/Shade");
				mCreditsTrayMgr->getBackdropLayer()->setZOrder(410);
				mCreditsTrayMgr->getTraysLayer()->setZOrder(411);
				
				float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
				Ogre::Viewport* vp = mWindow->getViewport(0);
				f *= (jNeedsUseLowResOverlays()) ? 1.f : 0.75f;
				
				OgreBites::DecorWidget* dlgBack = mCreditsTrayMgr->createDecorWidget(TL_CENTER, "CreditsBackPanel", "Rails/BlackOpaquePanel", f);
				Ogre::BorderPanelOverlayElement* panelEl = (Ogre::BorderPanelOverlayElement*)dlgBack->getOverlayElement();
				if (jNeedsUseLowResTextures())
					panelEl->setBorderSize(20, 20, 20, 20);
				dlgBack->getOverlayElement()->setDimensions(850 * f, 900 * f);
				mCreditsTrayMgr->adjustTrays();
				
				OgreBites::Label* lbl = mCreditsTrayMgr->createLabel(TL_NONE, "CreditsCaption", "Rails", dlgBack->getOverlayElement()->getWidth() * 0.8, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(midCaptionSize() * 0.9, "BCaption"));
	  			lbl->setCharHeight(midCaptionSize() * 0.9);
	  			lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
	  			lbl->getOverlayElement()->setPosition(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 2.0,
													(vp->getActualHeight() - dlgBack->getOverlayElement()->getHeight()) / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.04f);
				
				std::string version = jGetVersionName();
				lbl = mCreditsTrayMgr->createLabel(TL_NONE, "CreditsVersion", version, dlgBack->getOverlayElement()->getWidth() * 0.8, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(midCaptionSize() * 0.5, "Caption"));
	  			lbl->setCharHeight(midCaptionSize() * 0.5);
	  			lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
	  			lbl->getOverlayElement()->setPosition(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 2.0,
													(vp->getActualHeight() - dlgBack->getOverlayElement()->getHeight()) / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.04f + midCaptionSize() * 1.1);
				
				lbl = mCreditsTrayMgr->createLabel(TL_NONE, "CreditsDescription", "", dlgBack->getOverlayElement()->getWidth() * 0.9, true);
				lbl->getTextAreaElement()->setFontName(getFontNameForSize(midCaptionSize() * 0.74, "Caption"));
				lbl->setCharHeight(midCaptionSize() * 0.74);
				lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
				lbl->setWrappedText(jGetLocalizedString("credits.description"));
				lbl->getOverlayElement()->setHeight(dlgBack->getOverlayElement()->getHeight() * 0.8);
				lbl->getOverlayElement()->setLeft(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 2.f);
				//lbl->getOverlayElement()->setTop(vp->getActualHeight() / 2.f - lbl->getOverlayElement()->getHeight() * 0.8);
				int cnt = lbl->getLinesCount();
				float offset = lbl->getCharHeight() * cnt / 2.f;
				lbl->getOverlayElement()->setTop((vp->getActualHeight() - dlgBack->getOverlayElement()->getHeight()) / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.56f - offset);
				lbl->getTextAreaElement()->setTop( - lbl->getOverlayElement()->getHeight() / 2.f);
			}
		}
		
		void setupAchievementsTrayMgr()
		{
			if (!mAchTrayMgr)
			{
				mAchTrayMgr = new SdkTrayManager("MMAchievements_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mAchTrayMgr->hasWidget("AchBackPanel"))
			{
				mAchViewScrollPercentage = 0;
				//mAchTrayMgr->setBackdropMaterial(getChaptersBackgroundMatName());
				mAchTrayMgr->setBackdropMaterial("SdkTrays/Shade");
				mAchTrayMgr->getBackdropLayer()->setZOrder(410);
				mAchTrayMgr->getTraysLayer()->setZOrder(411);
				
				float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
				
				// OgreBites::DecorWidget* dlgBack = mAchTrayMgr->createDecorWidget(TL_CENTER, "AchBackPanel", "Rails/PrelevelSmallIcon", f);
				// dlgBack->getOverlayElement()->setMaterialName("SdkTrays/Empty");
				// dlgBack->getOverlayElement()->setDimensions(mWindow->getViewport(0)->getActualWidth(), mWindow->getViewport(0)->getActualHeight());
				// mAchTrayMgr->adjustTrays();
				bool lowResTextures = jNeedsUseLowResTextures();
				
				OgreBites::DecorWidget* dlgBack = mAchTrayMgr->createDecorWidget(TL_CENTER, "AchBackPanel", "Rails/BlackOpaquePanel", f);
				Ogre::BorderPanelOverlayElement* panelEl = (Ogre::BorderPanelOverlayElement*)dlgBack->getOverlayElement();
				if (lowResTextures)
				{
					panelEl->setBorderSize(20, 20, 20, 20);
					dlgBack->getOverlayElement()->setDimensions(mWindow->getViewport(0)->getActualWidth() * 0.7, mWindow->getViewport(0)->getActualHeight() + 20);
				}
				else
					dlgBack->getOverlayElement()->setDimensions(mWindow->getViewport(0)->getActualWidth() * 0.7, mWindow->getViewport(0)->getActualHeight() + 40);
				mAchTrayMgr->adjustTrays();
				
				// float ff = (float)mWindow->getViewport(0)->getActualHeight() / 1536.f;
				// ff *= 1.2;
				// OgreBites::Button* btn = mAchTrayMgr->createStartMenuButton(TL_BOTTOMLEFT, "Rails/BackButton", "AchievementsBackBtn", "", ff);
			
				mAchievementsBox = (Ogre::BorderPanelOverlayElement*)dlgBack->getOverlayElement();
			
				float elWidth = mAchievementsBox->getWidth() * 0.9;//mWindow->getViewport(0)->getActualWidth() * 0.7;
				float elHeight = elWidth / 8.f;
				int achCount = COUNT_ACHIEVEMENTS;
				int size = elHeight;
				int tLinesCount = 0;
				int dLinesCount = 0;
				float textWidth = elWidth - size * 1.1;//e->getWidth() * 0.8;
				float charHeight = elHeight * 0.25;
				
				mAchViewHeight = 0;
				
				//---
				Ogre::OverlayElement* e = (Ogre::OverlayElement*)Ogre::OverlayManager::getSingleton().createOverlayElementFromTemplate("Rails/AchievementItem", "", "AchItemsTitle");
				e->setDimensions(elWidth, elHeight);
				
				Ogre::OverlayElement* icon = (Ogre::OverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementIcon");
				icon->setMaterialName("SdkTrays/Empty");
				
				Ogre::TextAreaOverlayElement* title = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementTitle");
				// title->setAlignment(Ogre::TextAreaOverlayElement::Left);
				// title->setHorizontalAlignment(Ogre::GHA_LEFT);
				title->setFontName(getFontNameForSize(charHeight * 2.f, "BCaption"));
				title->setCharHeight(charHeight * 2.f);
				title->setWidth(elWidth);
				title->setHeight(e->getHeight());
				title->setCaption(jGetLocalizedString("Achievements"));
				title->setLeft(0);
				title->setTop( - charHeight / 2.f );
				
				Ogre::TextAreaOverlayElement* desc = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementDesc");
				desc->setCaption("");
				
				e->setPosition( - e->getWidth() * 0.5, 0);
				
				mAchViewHeight += e->getHeight() + 10;
				mAchievementsBox->addChild(e);
				mAchievementsItems.push_back(e);
				//---
			
				for (int idx = 0; idx < achCount; idx++)
				{
					bool isCompleted = CAchievements::shared()->isCompleted(idx);
					
					Ogre::OverlayElement* e = (Ogre::OverlayElement*)Ogre::OverlayManager::getSingleton().createOverlayElementFromTemplate("Rails/AchievementItem", "", "AchItem_" + Ogre::StringConverter::toString(idx));
					e->setDimensions(elWidth, elHeight);
					
					Ogre::OverlayElement* icon = (Ogre::OverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementIcon");
					icon->setMaterialName("Achievements/achievement_" + Ogre::StringConverter::toString(idx) + (isCompleted ? "" : "_closed"));
					icon->setDimensions(size, size);
					
					/*
					Ogre::TextAreaOverlayElement* progress = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementProgress");
					progress->setCharHeight(charHeight);
					//progress->setHorizontalAlignment(Ogre::GHA_LEFT);
					progress->setLeft(- elWidth / 2.0 + icon->getWidth() * 0.55);
					progress->setTop( - progress->getHeight() * 1.4);
					Ogre::String percents("");
					if (!isCompleted)
					{
						int countParts = CAchievements::shared()->countParts(idx);
						int completedParts = CAchievements::shared()->completedParts(idx);
	
						if (completedParts > 0)
						{
							percents = Ogre::StringConverter::toString((int)round((double)completedParts/(double)countParts*100.));
						}
						else
							percents = Ogre::StringConverter::toString((int)0);
						
						percents += "\%";
					}
					progress->setCaption(percents);
					*/
					
					Ogre::TextAreaOverlayElement* title = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementTitle");
					title->setAlignment(Ogre::TextAreaOverlayElement::Left);
					title->setHorizontalAlignment(Ogre::GHA_LEFT);
					title->setFontName(getFontNameForSize(charHeight, "BCaption"));
					title->setCharHeight(charHeight);
					title->setWidth(textWidth);
					title->setHeight(1);
					OgreBites::Label::setWrappedText(Ogre::UTFString(CAchievements::shared()->localizedName(idx)), title, textWidth, tLinesCount);
					title->setLeft(icon->getWidth() * 1.1);
					//title->setTop( - title->getHeight() * 1.0);
					
				
					Ogre::TextAreaOverlayElement* desc = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementDesc");
					desc->setAlignment(Ogre::TextAreaOverlayElement::Left);
					desc->setHorizontalAlignment(Ogre::GHA_LEFT);
					desc->setFontName(getFontNameForSize(charHeight, "Caption"));
					desc->setCharHeight(charHeight);
					desc->setWidth(textWidth);
					OgreBites::Label::setWrappedText(Ogre::UTFString(isCompleted ? CAchievements::shared()->localizedPostEarnedText(idx) : CAchievements::shared()->localizedPreEarnedText(idx)), desc, textWidth, dLinesCount);
					desc->setHeight(1);
					desc->setLeft(icon->getWidth() * 1.1);
					//desc->setTop( - desc->getHeight() * 0.1);
					
					float topOffset = - (tLinesCount + dLinesCount) * charHeight / 2.f;
					title->setTop(topOffset );
					desc->setTop(topOffset + (tLinesCount * charHeight));
				
					e->setPosition( - e->getWidth() * 0.5, (e->getHeight() + 10) * (idx + 1));
					mAchViewHeight += e->getHeight() + 10;
								
					mAchievementsBox->addChild(e);
					mAchievementsItems.push_back(e);
				}
				
				float pnlHeight = 24;
				if (lowResTextures)
					pnlHeight = 12;
				
				OgreBites::DecorWidget* top = mAchTrayMgr->createDecorWidget(TL_NONE, "AchBackPanelTop", "Rails/PrelevelBigIcon");
				top->getOverlayElement()->setDimensions(elWidth, pnlHeight);
				((Ogre::PanelOverlayElement*)top->getOverlayElement())->setUV(0.4, 0, 0.6, 0.24);
				top->setMaterial("Rails/BlackOpaquePanelMat");
				top->getOverlayElement()->setLeft((mWindow->getViewport(0)->getActualWidth() - top->getOverlayElement()->getWidth()) / 2.f );
				if (lowResTextures)
					top->getOverlayElement()->setTop(-10);
				else
					top->getOverlayElement()->setTop(-20);
				
				OgreBites::DecorWidget* bottom = mAchTrayMgr->createDecorWidget(TL_NONE, "AchBackPanelBottom", "Rails/PrelevelBigIcon");
				bottom->getOverlayElement()->setDimensions(elWidth, pnlHeight);
				((Ogre::PanelOverlayElement*)bottom->getOverlayElement())->setUV(0.4, 0.76, 0.6, 1);
				bottom->setMaterial("Rails/BlackOpaquePanelMat");
				bottom->getOverlayElement()->setLeft((mWindow->getViewport(0)->getActualWidth() - bottom->getOverlayElement()->getWidth()) / 2.f );
				if (lowResTextures)
					bottom->getOverlayElement()->setTop(mWindow->getViewport(0)->getActualHeight() - 2);
				else
					bottom->getOverlayElement()->setTop(mWindow->getViewport(0)->getActualHeight() - 4);
			}
			else
			{
				int achCount = COUNT_ACHIEVEMENTS;
				int tLinesCount = 0;
				int dLinesCount = 0;
				
				for (int idx = 0; idx < achCount; idx++)
				{
					int pos = idx + 1;
					if (pos > mAchievementsItems.size())
						break;
					
					bool isCompleted = CAchievements::shared()->isCompleted(idx);
					
					Ogre::OverlayElement* e = mAchievementsItems[pos];
					
					Ogre::OverlayElement* icon = (Ogre::OverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementIcon");
					icon->setMaterialName("Achievements/achievement_" + Ogre::StringConverter::toString(idx) + (isCompleted ? "" : "_closed"));
					
					Ogre::TextAreaOverlayElement* title = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementTitle");
					OgreBites::Label::setWrappedText(Ogre::UTFString(CAchievements::shared()->localizedName(idx)), title, title->getWidth(), tLinesCount);
					
					Ogre::TextAreaOverlayElement* desc = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/AchievementDesc");
					OgreBites::Label::setWrappedText(Ogre::UTFString(isCompleted ? CAchievements::shared()->localizedPostEarnedText(idx) : CAchievements::shared()->localizedPreEarnedText(idx)), desc, desc->getWidth(), dLinesCount);
					
					float topOffset = - (tLinesCount + dLinesCount) * title->getCharHeight() / 2.f;
					title->setTop(topOffset);
					desc->setTop(topOffset + (tLinesCount * title->getCharHeight()));
				}
			}
		}
		
		void setupLevelLockedTrayMgr()
		{
			if (!mLevelLockedTrayMgr)
			{
				mLevelLockedTrayMgr = new SdkTrayManager("MMLevelLocked_TrayMgr", mWindow, mInputContext, this);
			}
			
			if (!mLevelLockedTrayMgr->hasWidget("LevelLockedBackPanel"))
			{
				mLevelLockedTrayMgr->setBackdropMaterial("SdkTrays/Shade");
				mLevelLockedTrayMgr->getBackdropLayer()->setZOrder(410);
				mLevelLockedTrayMgr->getTraysLayer()->setZOrder(411);
				
				float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
				Ogre::Viewport* vp = mWindow->getViewport(0);
				f *= (jNeedsUseLowResOverlays()) ? 1.f : 0.75f;
				
				OgreBites::DecorWidget* dlgBack = mLevelLockedTrayMgr->createDecorWidget(TL_CENTER, "LevelLockedBackPanel", "Rails/BlackOpaquePanel", f);
				Ogre::BorderPanelOverlayElement* panelEl = (Ogre::BorderPanelOverlayElement*)dlgBack->getOverlayElement();
				if (jNeedsUseLowResTextures())
					panelEl->setBorderSize(20, 20, 20, 20);
				dlgBack->getOverlayElement()->setDimensions(900 * f, 700 * f);
				mLevelLockedTrayMgr->adjustTrays();
				
				OgreBites::Label* lbl = NULL;
				
				OgreBites::DecorWidget* achievement = mLevelLockedTrayMgr->createDecorWidget(TL_NONE, "LevelLockedAchievementIcon", "Rails/PrelevelBigIcon");
				achievement->getOverlayElement()->setDimensions(250 * f, 250 * f);
	  			achievement->getOverlayElement()->setPosition(vp->getActualWidth() / 2.0 - dlgBack->getOverlayElement()->getWidth() * 0.32 - achievement->getOverlayElement()->getWidth() * 0.25,
															 vp->getActualHeight() / 2.0 + achievement->getOverlayElement()->getHeight() * 0.05f);
				achievement->setMaterial("Achievements/achievement_1");
				
				lbl = mLevelLockedTrayMgr->createLabel(TL_NONE, "LevelLocked_Caption", "", dlgBack->getOverlayElement()->getWidth() * 0.8, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(midCaptionSize(), "BCaption"));
	  			lbl->setCharHeight(midCaptionSize());
	  			lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
				lbl->setWrappedText(jGetLocalizedString("To_unlock_this_level_you_must_complete_the_following_achievement"));
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
	  			lbl->getOverlayElement()->setLeft(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 2.0);
				lbl->getOverlayElement()->setTop((vp->getActualHeight() - dlgBack->getOverlayElement()->getHeight()) / 2.f + dlgBack->getOverlayElement()->getHeight() * 0.06f);
				
				lbl = mLevelLockedTrayMgr->createLabel(TL_NONE, "LevelLocked_AchTitle", "", dlgBack->getOverlayElement()->getWidth() * 0.5, true);
	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(midCaptionSize() * 0.84, "BICaption"));
				lbl->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
				lbl->getTextAreaElement()->setHorizontalAlignment(Ogre::GHA_LEFT);
	  			lbl->setCharHeight(midCaptionSize() * 0.84);
	  			lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
				lbl->setWrappedText(jGetLocalizedString("To_unlock_this_level_you_must_complete_the_following_achievement"));
	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
	  			lbl->getOverlayElement()->setPosition(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() * 0.1f, 0);
				lbl->getTextAreaElement()->setTop( - lbl->getOverlayElement()->getHeight() / 2.f);
													  
  				lbl = mLevelLockedTrayMgr->createLabel(TL_NONE, "LevelLocked_AchDescription", "", dlgBack->getOverlayElement()->getWidth() * 0.5, true);
  	  			lbl->getTextAreaElement()->setFontName(getFontNameForSize(midCaptionSize() * 0.75, "Caption"));
  				lbl->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
  				lbl->getTextAreaElement()->setHorizontalAlignment(Ogre::GHA_LEFT);
  	  			lbl->setCharHeight(midCaptionSize() * 0.75);
  	  			lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
  				lbl->setWrappedText(jGetLocalizedString("To_unlock_this_level_you_must_complete_the_following_achievement"));
  	  			lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
  	  			lbl->getOverlayElement()->setPosition(vp->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() * 0.1f, 0);
				lbl->getTextAreaElement()->setTop( - lbl->getOverlayElement()->getHeight() / 2.f);
													
			}
			
			
		}
		
		void updateAchScrollView()
		{
			float scrollOffset = mAchViewScrollPercentage * (mAchViewHeight - mWindow->getViewport(0)->getActualHeight());
			for (int idx = 0; idx < mAchievementsItems.size(); idx++)
			{
				Ogre::OverlayElement* e = mAchievementsItems[idx];
				e->setTop((e->getHeight() + 10) * idx - scrollOffset);
			}
		}
		
		float getAchScrollPercentage()
		{
			return mAchViewScrollPercentage;
		}
		
		void setAchScrollPercentage(Ogre::Real percentage)
		{
			mAchViewScrollPercentage = Ogre::Math::Clamp<Ogre::Real>(percentage, 0, 1);
			//mScrollHandle->setTop((int)(percentage * (mScrollTrack->getHeight() - mScrollHandle->getHeight())));
			updateAchScrollView();
		}
		
		SdkTrayManager* mTrayMgr;
		SdkTrayManager* mChaptersTrayMgr;
		SdkTrayManager* mChapter1TrayMgr;
		SdkTrayManager* mChapter2TrayMgr;
		SdkTrayManager* mChapter3TrayMgr;
		SdkTrayManager* mAchTrayMgr;
		SdkTrayManager* mCreditsTrayMgr;
		SdkTrayManager* mOptionsTrayMgr;
		SdkTrayManager* mLevelLockedTrayMgr;
		SdkTrayManager* mHighScoresTrayMgr;
		SdkTrayManager* mUnlockChaptersOfferTrayMgr;
		
		SdkTrayManager* mDebugTrayMgr;
		
		bool mAllChaptersUnlocked;
		bool mUnlockLevelsLockedByAchievemints;
		bool mShowRandomAchievementsInCompleteLevelView;
		
		Ogre::BorderPanelOverlayElement* mHighScoresBox;
		std::vector<Ogre::OverlayElement*> mHighScoresItems;
		Ogre::BorderPanelOverlayElement* mAchievementsBox;
		std::vector<Ogre::OverlayElement*> mAchievementsItems;
		float mAchViewHeight;
		float mAchViewScrollPercentage;
		float mAchMaxOffset;
		float mAchSign;
		float mAchLastOffsetY;
		bool mIsAchScrolling;
		Ogre::Vector2 mAchPressedCoord;
		timespec mAchScrollStart;
		timespec mAchScrollEnd;
		
		Ogre::Root* mRoot;              // OGRE root
		OIS::InputManager* mInputMgr;   // OIS input manager
		InputContext mInputContext;		// all OIS devices are here
		Ogre::OverlaySystem* mOverlaySystem;  // Overlay system
		COgreScene* mCurrentScene;         // currently running Scene
		bool mScenePaused;             // whether current Scene is paused
		bool mFirstRun;                 // whether or not this is the first run
		bool mLastRun;                  // whether or not this is the final run
		
		bool mIsTrainopediaOnScreen;
		
		OgreBites::GameButton* mEraseButton;
		timespec mEraseButtonPressedTime;
		bool mEraseButtonPressed;
		
		int _smallFontSize;
		int _midFontSize;
		int _bigFontSize;
		bool _bigFontEnabled;
		
		OgreBites::Button* mStartGameBtn;
		
#ifdef USE_RTSHADER_SYSTEM
		Ogre::RTShader::ShaderGenerator*			mShaderGenerator;			// The Shader generator instance.
		ShaderGeneratorTechniqueResolverListener*	mMaterialMgrListener;		// Shader generator material manager listener.	
#endif // USE_RTSHADER_SYSTEM
		
    public:
        Ogre::RenderWindow* mWindow;    // render window
	};
}

#endif
